@@ -32,8 +32,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "sudo_compat.h"
-#include "sudo_util.h"
+#include <sudo_compat.h>
+#include <sudo_util.h>
 
 # if defined(__linux__)
 /* 
@@ -44,7 +44,7 @@
  * Returns a dynamically allocated string on success and NULL on failure.
  */
 char *
-sudo_stat_multiarch_v1(const char *path, struct stat *sb)
+sudo_stat_multiarch_v1(const char * restrict path, struct stat * restrict sb)
 {
 #  if defined(__ILP32__)
     const char *libdirs[] = { "/libx32/", "/lib/", "/libexec/", NULL };
@@ -56,7 +56,7 @@ sudo_stat_multiarch_v1(const char *path, struct stat *sb)
     const char **lp, *lib, *slash;
     struct utsname unamebuf;
     char *newpath = NULL;
-    int len;
+    size_t len;
 
     if (uname(&unamebuf) == -1)
 	return NULL;
@@ -79,9 +79,8 @@ sudo_stat_multiarch_v1(const char *path, struct stat *sb)
 	}
 
 	/* Add machine-linux-gnu dir after /lib/ or /libexec/. */
-	len = asprintf(&newpath, "%.*s%s%s-linux-gnu%s",
-	    (int)(lib - path), path, newlib, unamebuf.machine, slash);
-	if (len == -1) {
+	if (asprintf(&newpath, "%.*s%s%s-linux-gnu%s",
+	    (int)(lib - path), path, newlib, unamebuf.machine, slash) == -1) {
 	    newpath = NULL;
 	    break;
 	}
@@ -97,7 +96,7 @@ sudo_stat_multiarch_v1(const char *path, struct stat *sb)
 }
 #else
 char *
-sudo_stat_multiarch_v1(const char *path, struct stat *sb)
+sudo_stat_multiarch_v1(const char * restrict path, struct stat * restrict sb)
 {
     return NULL;
 }
