@@ -30,7 +30,7 @@
 
 #include <string.h>
 
-#include "sudo_compat.h"
+#include <sudo_compat.h>
 
 /*
  * Copy string src to buffer dst of size dsize.  At most dsize-1
@@ -38,7 +38,7 @@
  * Returns strlen(src); if retval >= dsize, truncation occurred.
  */
 size_t
-sudo_strlcpy(char *dst, const char *src, size_t dsize)
+sudo_strlcpy(char * restrict dst, const char * restrict src, size_t dsize)
 {
 	const char *osrc = src;
 	size_t nleft = dsize;
@@ -59,6 +59,6 @@ sudo_strlcpy(char *dst, const char *src, size_t dsize)
 			continue;
 	}
 
-	return(src - osrc - 1);	/* count does not include NUL */
+	return((size_t)(src - osrc) - 1);	/* count does not include NUL */
 }
 #endif /* HAVE_STRLCPY */
