@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 1993-1996,1998-2005, 2007-2018
+ * Copyright (c) 1993-1996,1998-2005, 2007-2024
  *	Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -39,81 +39,133 @@
 #include <pwd.h>
 #include <grp.h>
 
-#include "sudoers.h"
-#include "check.h"
-
-struct getpass_closure {
-    int tstat;
-    int lectured;
-    void *cookie;
-    struct passwd *auth_pw;
-};
-
-static struct passwd *get_authpw(int);
+#include <sudoers.h>
+#include <timestamp.h>
 
 /*
- * Called when getpass is suspended so we can drop the lock.
+ * Get passwd entry for the user we are going to authenticate as.
+ * By default, this is the user invoking sudo.  In the most common
+ * case, this matches ctx->user.pw or ctx->runas.pw.
  */
-static int
-getpass_suspend(int signo, void *vclosure)
+static struct passwd *
+get_authpw(struct sudoers_context *ctx, unsigned int mode)
 {
-    struct getpass_closure *closure = vclosure;
+    struct passwd *pw = NULL;
+    debug_decl(get_authpw, SUDOERS_DEBUG_AUTH);
 
-    timestamp_close(closure->cookie);
-    closure->cookie = NULL;
-    return 0;
+    if (ISSET(mode, (MODE_CHECK|MODE_LIST))) {
+	/* In list mode we always prompt for the user's password. */
+	sudo_pw_addref(ctx->user.pw);
+	pw = ctx->user.pw;
+    } else {
+	if (def_rootpw) {
+	    if ((pw = sudo_getpwuid(ROOT_UID)) == NULL) {
+		log_warningx(ctx, SLOG_SEND_MAIL, N_("unknown uid %u"),
+		    ROOT_UID);
+	    }
+	} else if (def_runaspw) {
+	    if ((pw = sudo_getpwnam(def_runas_default)) == NULL) {
+		log_warningx(ctx, SLOG_SEND_MAIL,
+		    N_("unknown user %s"), def_runas_default);
+	    }
+	} else if (def_targetpw) {
+	    if (ctx->runas.pw->pw_name == NULL) {
+		/* This should never be NULL as we fake up the passwd struct */
+		log_warningx(ctx, SLOG_RAW_MSG, N_("unknown uid %u"),
+		    (unsigned int) ctx->runas.pw->pw_uid);
+	    } else {
+		sudo_pw_addref(ctx->runas.pw);
+		pw = ctx->runas.pw;
+	    }
+	} else {
+	    sudo_pw_addref(ctx->user.pw);
+	    pw = ctx->user.pw;
+	}
+    }
+
+    debug_return_ptr(pw);
 }
 
 /*
- * Called when getpass is resumed so we can reacquire the lock.
+ * Returns AUTH_SUCCESS if the user successfully authenticates,
+ * AUTH_FAILURE if not or AUTH_ERROR on error.
  */
-static int
-getpass_resume(int signo, void *vclosure)
-{
-    struct getpass_closure *closure = vclosure;
-
-    closure->cookie = timestamp_open(user_name, user_sid);
-    if (closure->cookie == NULL)
-	return -1;
-    if (!timestamp_lock(closure->cookie, closure->auth_pw))
-	return -1;
-    return 0;
-}
-
-/*
- * Returns true if the user successfully authenticates, false if not
- * or -1 on fatal error.
- */
-static int
-check_user_interactive(int validated, int mode, struct getpass_closure *closure)
+int
+check_user(struct sudoers_context *ctx, unsigned int validated,
+    unsigned int mode)
 {
+    struct getpass_closure closure = { 0 };
     struct sudo_conv_callback callback;
-    int ret = -1;
+    int status = TS_ERROR;
+    int ret = AUTH_ERROR;
+    bool exempt = false;
     char *prompt;
-    debug_decl(check_user_interactive, SUDOERS_DEBUG_AUTH);
+    debug_decl(check_user, SUDOERS_DEBUG_AUTH);
+
+    /*
+     * In intercept mode, only check the user if configured to do so.
+     * We already have a session so no need to init the auth subsystem.
+     */
+    if (ISSET(ctx->mode, MODE_POLICY_INTERCEPTED)) {
+	if (!def_intercept_authenticate) {
+	    debug_return_int(AUTH_SUCCESS);
+	}
+    }
+
+    /*
+     * Init authentication system regardless of whether we need a password.
+     * Required for proper PAM session support.
+     */
+    if ((closure.auth_pw = get_authpw(ctx, mode)) == NULL)
+	debug_return_int(AUTH_ERROR);
+    if (sudo_auth_init(ctx, closure.auth_pw, mode) != AUTH_SUCCESS) {
+	sudo_pw_delref(closure.auth_pw);
+	debug_return_int(AUTH_ERROR);
+    }
+    closure.ctx = ctx;
+
+    /*
+     * Don't prompt for the root passwd or if the user is exempt.
+     * If the user is not changing uid/gid, no need for a password.
+     */
+    if (!def_authenticate || user_is_exempt(ctx)) {
+	sudo_debug_printf(SUDO_DEBUG_INFO, "%s: %s", __func__,
+	    !def_authenticate ? "authentication disabled" :
+	    "user exempt from authentication");
+	exempt = true;
+	ret = AUTH_SUCCESS;
+	goto done;
+    }
+    if (ctx->user.uid == 0 || (ctx->user.uid == ctx->runas.pw->pw_uid &&
+	(ctx->runas.gr == NULL ||
+	user_in_group(ctx->user.pw, ctx->runas.gr->gr_name)))) {
+	if (ctx->runas.role == NULL && ctx->runas.type == NULL &&
+	    ctx->runas.apparmor_profile == NULL &&
+	    ctx->runas.privs == NULL && ctx->runas.limitprivs == NULL) {
+	    sudo_debug_printf(SUDO_DEBUG_INFO,
+		"%s: user running command as self", __func__);
+	    ret = AUTH_SUCCESS;
+	    goto done;
+	}
+    }
 
     /* Construct callback for getpass function. */
     memset(&callback, 0, sizeof(callback));
     callback.version = SUDO_CONV_CALLBACK_VERSION;
-    callback.closure = closure;
-    callback.on_suspend = getpass_suspend;
-    callback.on_resume = getpass_resume;
+    callback.closure = &closure;
 
     /* Open, lock and read time stamp file if we are using it. */
     if (!ISSET(mode, MODE_IGNORE_TICKET)) {
 	/* Open time stamp file and check its status. */
-	closure->cookie = timestamp_open(user_name, user_sid);
-	if (closure->cookie != NULL) {
-	    if (timestamp_lock(closure->cookie, closure->auth_pw)) {
-		closure->tstat = timestamp_status(closure->cookie,
-		    closure->auth_pw);
+	closure.cookie = timestamp_open(ctx);
+	if (closure.cookie != NULL) {
+	    if (timestamp_lock(closure.cookie, closure.auth_pw)) {
+		status = timestamp_status(closure.cookie, closure.auth_pw);
 	    }
-	    callback.on_suspend = getpass_suspend;
-	    callback.on_resume = getpass_resume;
 	}
     }
 
-    switch (closure->tstat) {
+    switch (status) {
     case TS_FATAL:
 	/* Fatal error (usually setuid failure), unsafe to proceed. */
 	goto done;
@@ -121,7 +173,7 @@ check_user_interactive(int validated, int mode, struct getpass_closure *closure)
     case TS_CURRENT:
 	/* Time stamp file is valid and current. */
 	if (!ISSET(validated, FLAG_CHECK_USER)) {
-	    ret = true;
+	    ret = AUTH_SUCCESS;
 	    break;
 	}
 	sudo_debug_printf(SUDO_DEBUG_INFO,
@@ -131,109 +183,41 @@ check_user_interactive(int validated, int mode, struct getpass_closure *closure)
     default:
 	if (ISSET(mode, MODE_NONINTERACTIVE) && !def_noninteractive_auth) {
 	    validated |= FLAG_NO_USER_INPUT;
-	    log_auth_failure(validated, 0);
+	    log_auth_failure(ctx, validated, 0);
 	    goto done;
 	}
 
 	/* Expand any escapes in the prompt. */
-	prompt = expand_prompt(user_prompt ? user_prompt : def_passprompt,
-	    closure->auth_pw->pw_name);
+	prompt = expand_prompt(ctx,
+	    ctx->user.prompt ? ctx->user.prompt : def_passprompt,
+	    closure.auth_pw->pw_name);
 	if (prompt == NULL)
 	    goto done;
 
-	ret = verify_user(closure->auth_pw, prompt, validated, &callback);
-	if (ret == true && closure->lectured)
-	    (void)set_lectured();	/* lecture error not fatal */
+	ret = verify_user(ctx, closure.auth_pw, prompt, validated, &callback);
+	if (ret == AUTH_SUCCESS && closure.lectured)
+	    (void)set_lectured(ctx);	/* lecture error not fatal */
 	free(prompt);
 	break;
     }
 
 done:
-    debug_return_int(ret);
-}
-
-/*
- * Returns true if the user successfully authenticates, false if not
- * or -1 on error.
- */
-int
-check_user(int validated, int mode)
-{
-    struct getpass_closure closure = { TS_ERROR };
-    int ret = -1;
-    bool exempt = false;
-    debug_decl(check_user, SUDOERS_DEBUG_AUTH);
-
-    /*
-     * In intercept mode, only check the user if configured to do so.
-     * We already have a session so no need to init the auth subsystem.
-     */
-    if (ISSET(sudo_mode, MODE_POLICY_INTERCEPTED)) {
-	if (!def_intercept_authenticate) {
-	    debug_return_int(true);
-	}
-    }
-
-    /*
-     * Init authentication system regardless of whether we need a password.
-     * Required for proper PAM session support.
-     */
-    if ((closure.auth_pw = get_authpw(mode)) == NULL)
-	goto done;
-    if (sudo_auth_init(closure.auth_pw, mode) == -1)
-	goto done;
-
-    /*
-     * Don't prompt for the root passwd or if the user is exempt.
-     * If the user is not changing uid/gid, no need for a password.
-     */
-    if (!def_authenticate || user_is_exempt()) {
-	sudo_debug_printf(SUDO_DEBUG_INFO, "%s: %s", __func__,
-	    !def_authenticate ? "authentication disabled" :
-	    "user exempt from authentication");
-	exempt = true;
-	ret = true;
-	goto done;
-    }
-    if (user_uid == 0 || (user_uid == runas_pw->pw_uid &&
-	(!runas_gr || user_in_group(sudo_user.pw, runas_gr->gr_name)))) {
-#ifdef HAVE_SELINUX
-	if (user_role == NULL && user_type == NULL)
-#endif
-#ifdef HAVE_APPARMOR
-	if (user_apparmor_profile == NULL)
-#endif
-#ifdef HAVE_PRIV_SET
-	if (runas_privs == NULL && runas_limitprivs == NULL)
-#endif
-	{
-	    sudo_debug_printf(SUDO_DEBUG_INFO,
-		"%s: user running command as self", __func__);
-	    ret = true;
-	    goto done;
-	}
-    }
-
-    ret = check_user_interactive(validated, mode, &closure);
-
-done:
-    if (ret == true) {
+    if (ret == AUTH_SUCCESS) {
 	/* The approval function may disallow a user post-authentication. */
-	ret = sudo_auth_approval(closure.auth_pw, validated, exempt);
+	ret = sudo_auth_approval(ctx, closure.auth_pw, validated, exempt);
 
 	/*
 	 * Only update time stamp if user validated and was approved.
 	 * Failure to update the time stamp is not a fatal error.
 	 */
-	if (ret == true && ISSET(validated, VALIDATE_SUCCESS)) {
-	    if (ISSET(mode, MODE_UPDATE_TICKET) && closure.tstat != TS_ERROR)
+	if (ret == AUTH_SUCCESS && ISSET(validated, VALIDATE_SUCCESS)) {
+	    if (ISSET(mode, MODE_UPDATE_TICKET) && status != TS_ERROR)
 		(void)timestamp_update(closure.cookie, closure.auth_pw);
 	}
     }
     timestamp_close(closure.cookie);
-    sudo_auth_cleanup(closure.auth_pw, !ISSET(validated, VALIDATE_SUCCESS));
-    if (closure.auth_pw != NULL)
-	sudo_pw_delref(closure.auth_pw);
+    sudo_auth_cleanup(ctx, closure.auth_pw, !ISSET(validated, VALIDATE_SUCCESS));
+    sudo_pw_delref(closure.auth_pw);
 
     debug_return_int(ret);
 }
@@ -260,7 +244,8 @@ display_lecture(struct sudo_conv_callback *callback)
     if (closure->lectured)
 	debug_return;
 
-    if (def_lecture == never || (def_lecture == once && already_lectured()))
+    if (def_lecture == never ||
+	    (def_lecture == once && already_lectured(closure->ctx)))
 	debug_return;
 
     memset(&msg, 0, sizeof(msg));
@@ -281,16 +266,16 @@ display_lecture(struct sudo_conv_callback *callback)
 		    close(fd);
 		    goto done;
 		}
-		log_warning(SLOG_RAW_MSG,
+		log_warning(closure->ctx, SLOG_RAW_MSG,
 		    N_("error reading lecture file %s"), def_lecture_file);
 	    } else {
-		log_warningx(SLOG_RAW_MSG,
+		log_warningx(closure->ctx, SLOG_RAW_MSG,
 		    N_("ignoring lecture file %s: not a regular file"),
 		    def_lecture_file);
 	    }
 	} else {
-	    log_warning(SLOG_RAW_MSG|SLOG_NO_STDERR, N_("unable to open %s"),
-		def_lecture_file);
+	    log_warning(closure->ctx, SLOG_RAW_MSG|SLOG_NO_STDERR,
+		N_("unable to open %s"), def_lecture_file);
 	}
 	if (fd != -1)
 	    close(fd);
@@ -320,82 +305,14 @@ done:
  * Checks if the user is exempt from supplying a password.
  */
 bool
-user_is_exempt(void)
+user_is_exempt(const struct sudoers_context *ctx)
 {
     bool ret = false;
     debug_decl(user_is_exempt, SUDOERS_DEBUG_AUTH);
 
     if (def_exempt_group) {
-	if (user_in_group(sudo_user.pw, def_exempt_group))
+	if (user_in_group(ctx->user.pw, def_exempt_group))
 	    ret = true;
     }
     debug_return_bool(ret);
 }
-
-/*
- * Get passwd entry for the user we are going to authenticate as.
- * By default, this is the user invoking sudo.  In the most common
- * case, this matches sudo_user.pw or runas_pw.
- */
-static struct passwd *
-get_authpw(int mode)
-{
-    struct passwd *pw = NULL;
-    debug_decl(get_authpw, SUDOERS_DEBUG_AUTH);
-
-    if (ISSET(mode, (MODE_CHECK|MODE_LIST))) {
-	/* In list mode we always prompt for the user's password. */
-	sudo_pw_addref(sudo_user.pw);
-	pw = sudo_user.pw;
-    } else {
-	if (def_rootpw) {
-	    if ((pw = sudo_getpwuid(ROOT_UID)) == NULL) {
-		log_warningx(SLOG_SEND_MAIL, N_("unknown uid %u"), ROOT_UID);
-	    }
-	} else if (def_runaspw) {
-	    if ((pw = sudo_getpwnam(def_runas_default)) == NULL) {
-		log_warningx(SLOG_SEND_MAIL,
-		    N_("unknown user %s"), def_runas_default);
-	    }
-	} else if (def_targetpw) {
-	    if (runas_pw->pw_name == NULL) {
-		/* This should never be NULL as we fake up the passwd struct */
-		log_warningx(SLOG_RAW_MSG, N_("unknown uid %u"),
-		    (unsigned int) runas_pw->pw_uid);
-	    } else {
-		sudo_pw_addref(runas_pw);
-		pw = runas_pw;
-	    }
-	} else {
-	    sudo_pw_addref(sudo_user.pw);
-	    pw = sudo_user.pw;
-	}
-    }
-
-    debug_return_ptr(pw);
-}
-
-/*
- * Returns true if the specified shell is allowed by /etc/shells, else false.
- */
-bool
-check_user_shell(const struct passwd *pw)
-{
-    const char *shell;
-    debug_decl(check_user_shell, SUDOERS_DEBUG_AUTH);
-
-    if (!def_runas_check_shell)
-	debug_return_bool(true);
-
-    sudo_debug_printf(SUDO_DEBUG_INFO,
-	"%s: checking /etc/shells for %s", __func__, pw->pw_shell);
-
-    setusershell();
-    while ((shell = getusershell()) != NULL) {
-	if (strcmp(shell, pw->pw_shell) == 0)
-	    debug_return_bool(true);
-    }
-    endusershell();
-
-    debug_return_bool(false);
-}
