@@ -35,7 +35,7 @@
 #include <unistd.h>
 #include <errno.h>
 
-#include "sudoers.h"
+#include <sudoers.h>
 
 /*
  * Check the given command against the specified allowlist (NULL-terminated).
@@ -78,16 +78,16 @@ cmnd_allowed(char *cmnd, size_t cmnd_size, struct stat *cmnd_sbp,
 }
 
 /*
- * This function finds the full pathname for a command and
- * stores it in a statically allocated array, filling in a pointer
- * to the array.  Returns FOUND if the command was found, NOT_FOUND
- * if it was not found, or NOT_FOUND_DOT if it would have been found
- * but it is in '.' and IGNORE_DOT is set.
+ * This function finds the full pathname for a command and stores it
+ * in a statically allocated array, filling in a pointer to the array.
+ * Returns FOUND if the command was found, NOT_FOUND if it was not found,
+ * NOT_FOUND_DOT if it would have been found but it is in '.' and
+ * ignore_dot is set or NOT_FOUND_ERROR if an error occurred.
  * The caller is responsible for freeing the output file.
  */
 int
 find_path(const char *infile, char **outfile, struct stat *sbp,
-    const char *path, int ignore_dot, char * const *allowlist)
+    const char *path, bool ignore_dot, char * const *allowlist)
 {
     char command[PATH_MAX];
     const char *cp, *ep, *pathend;
