@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 1994-1996, 1998-2022 Todd C. Miller <Todd.Miller@sudo.ws>
+ * Copyright (c) 1994-1996, 1998-2024 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -50,13 +50,13 @@
 #include <signal.h>
 #include <syslog.h>
 #ifndef HAVE_GETADDRINFO
-# include "compat/getaddrinfo.h"
+# include <compat/getaddrinfo.h>
 #endif
 
-#include "sudoers.h"
+#include <sudoers.h>
 #ifdef SUDOERS_LOG_CLIENT
-# include "log_client.h"
-# include "strlist.h"
+# include <log_client.h>
+# include <strlist.h>
 #endif
 
 struct parse_error {
@@ -67,7 +67,7 @@ STAILQ_HEAD(parse_error_list, parse_error);
 static struct parse_error_list parse_error_list =
     STAILQ_HEAD_INITIALIZER(parse_error_list);
 
-static bool should_mail(int);
+static bool should_mail(const struct sudoers_context *ctx, unsigned int);
 static bool warned = false;
 
 #ifdef SUDOERS_LOG_CLIENT
@@ -127,7 +127,8 @@ init_log_details(struct log_details *details, struct eventlog *evlog)
 }
 
 bool
-log_server_reject(struct eventlog *evlog, const char *message)
+log_server_reject(const struct sudoers_context *ctx, struct eventlog *evlog,
+    const char *message)
 {
     bool ret = false;
     debug_decl(log_server_reject, SUDOERS_DEBUG_LOGGING);
@@ -135,7 +136,7 @@ log_server_reject(struct eventlog *evlog, const char *message)
     if (SLIST_EMPTY(&def_log_servers))
 	debug_return_bool(true);
 
-    if (ISSET(sudo_mode, MODE_POLICY_INTERCEPTED)) {
+    if (ISSET(ctx->mode, MODE_POLICY_INTERCEPTED)) {
 	/* Older servers don't support multiple commands per session. */
 	if (!client_closure->subcommands)
 	    debug_return_bool(true);
@@ -156,8 +157,8 @@ log_server_reject(struct eventlog *evlog, const char *message)
 	    debug_return_bool(false);
 
 	/* Open connection to log server, send hello and reject messages. */
-	client_closure = log_server_open(&details, &sudo_user.submit_time,
-	    false, SEND_REJECT, message);
+	client_closure = log_server_open(&details, NULL, false,
+	    SEND_REJECT, message);
 	if (client_closure != NULL) {
 	    client_closure_free(client_closure);
 	    client_closure = NULL;
@@ -173,7 +174,7 @@ done:
 }
 
 bool
-log_server_alert(struct eventlog *evlog, struct timespec *now,
+log_server_alert(const struct sudoers_context *ctx, struct eventlog *evlog,
     const char *message, const char *errstr)
 {
     struct log_details details;
@@ -191,7 +192,7 @@ log_server_alert(struct eventlog *evlog, struct timespec *now,
 	}
     }
 
-    if (ISSET(sudo_mode, MODE_POLICY_INTERCEPTED)) {
+    if (ISSET(ctx->mode, MODE_POLICY_INTERCEPTED)) {
 	/* Older servers don't support multiple commands per session. */
 	if (!client_closure->subcommands) {
             ret = true;
@@ -199,7 +200,7 @@ log_server_alert(struct eventlog *evlog, struct timespec *now,
 	}
 
 	/* Use existing client closure. */
-        if (fmt_reject_message(client_closure, evlog)) {
+        if (fmt_alert_message(client_closure, evlog)) {
             if (client_closure->write_ev->add(client_closure->write_ev,
                     &client_closure->log_details->server_timeout) == -1) {
                 sudo_warn("%s", U_("unable to add event to queue"));
@@ -212,7 +213,7 @@ log_server_alert(struct eventlog *evlog, struct timespec *now,
 	    goto done;
 
 	/* Open connection to log server, send hello and alert messages. */
-	client_closure = log_server_open(&details, now, false,
+	client_closure = log_server_open(&details, NULL, false,
 	    SEND_ALERT, emessage ? emessage : message);
 	if (client_closure != NULL) {
 	    client_closure_free(client_closure);
@@ -230,13 +231,14 @@ done:
 }
 #else
 bool
-log_server_reject(struct eventlog *evlog, const char *message)
+log_server_reject(const struct sudoers_context *ctx, struct eventlog *evlog,
+    const char *message)
 {
     return true;
 }
 
 bool
-log_server_alert(struct eventlog *evlog, struct timespec *now,
+log_server_alert(const struct sudoers_context *ctx, struct eventlog *evlog,
     const char *message, const char *errstr)
 {
     return true;
@@ -247,7 +249,8 @@ log_server_alert(struct eventlog *evlog, struct timespec *now,
  * Log a reject event to syslog, a log file, sudo_logsrvd and/or email.
  */
 static bool
-log_reject(const char *message, bool logit, bool mailit)
+log_reject(const struct sudoers_context *ctx, const char *message,
+    bool logit, bool mailit)
 {
     const char *uuid_str = NULL;
     struct eventlog evlog;
@@ -255,17 +258,18 @@ log_reject(const char *message, bool logit, bool mailit)
     bool ret;
     debug_decl(log_reject, SUDOERS_DEBUG_LOGGING);
 
-    if (!ISSET(sudo_mode, MODE_POLICY_INTERCEPTED))
-	uuid_str = sudo_user.uuid_str;
+    if (!ISSET(ctx->mode, MODE_POLICY_INTERCEPTED))
+	uuid_str = ctx->uuid_str;
 
     if (mailit) {
 	SET(evl_flags, EVLOG_MAIL);
 	if (!logit)
 	    SET(evl_flags, EVLOG_MAIL_ONLY);
     }
-    sudoers_to_eventlog(&evlog, safe_cmnd, NewArgv, env_get(), uuid_str);
+    sudoers_to_eventlog(ctx, &evlog, ctx->runas.cmnd, ctx->runas.argv,
+	NULL, uuid_str);
     ret = eventlog_reject(&evlog, evl_flags, message, NULL, NULL);
-    if (!log_server_reject(&evlog, message))
+    if (!log_server_reject(ctx, &evlog, message))
 	ret = false;
 
     debug_return_bool(ret);
@@ -275,7 +279,8 @@ log_reject(const char *message, bool logit, bool mailit)
  * Log, audit and mail the denial message, optionally informing the user.
  */
 bool
-log_denial(int status, bool inform_user)
+log_denial(const struct sudoers_context *ctx, unsigned int status,
+    bool inform_user)
 {
     const char *message;
     int oldlocale;
@@ -283,24 +288,26 @@ log_denial(int status, bool inform_user)
     debug_decl(log_denial, SUDOERS_DEBUG_LOGGING);
 
     /* Send mail based on status. */
-    mailit = should_mail(status);
+    mailit = should_mail(ctx, status);
 
     /* Set error message. */
     if (ISSET(status, FLAG_NO_USER))
 	message = N_("user NOT in sudoers");
     else if (ISSET(status, FLAG_NO_HOST))
 	message = N_("user NOT authorized on host");
+    else if (ISSET(status, FLAG_INTERCEPT_SETID))
+	message = N_("setid command rejected in intercept mode");
     else
 	message = N_("command not allowed");
 
     /* Do auditing first (audit_failure() handles the locale itself). */
-    audit_failure(NewArgv, "%s", message);
+    audit_failure(ctx, ctx->runas.argv, "%s", message);
 
     if (def_log_denied || mailit) {
 	/* Log and mail messages should be in the sudoers locale. */
 	sudoers_setlocale(SUDOERS_LOCALE_SUDOERS, &oldlocale);
 
-	if (!log_reject(message, def_log_denied, mailit))
+	if (!log_reject(ctx, message, def_log_denied, mailit))
 	    ret = false;
 
 	/* Restore locale. */
@@ -313,21 +320,38 @@ log_denial(int status, bool inform_user)
 
 	if (ISSET(status, FLAG_NO_USER)) {
 	    sudo_printf(SUDO_CONV_ERROR_MSG, _("%s is not in the sudoers "
-		"file.\n"), user_name);
+		"file.\n"), ctx->user.name);
 	} else if (ISSET(status, FLAG_NO_HOST)) {
 	    sudo_printf(SUDO_CONV_ERROR_MSG, _("%s is not allowed to run sudo "
-		"on %s.\n"), user_name, user_srunhost);
+		"on %s.\n"), ctx->user.name, ctx->runas.shost);
+	} else if (ISSET(status, FLAG_INTERCEPT_SETID)) {
+	    sudo_printf(SUDO_CONV_ERROR_MSG, _("%s: %s\n"), getprogname(),
+		_("setid commands are not permitted in intercept mode"));
 	} else if (ISSET(status, FLAG_NO_CHECK)) {
 	    sudo_printf(SUDO_CONV_ERROR_MSG, _("Sorry, user %s may not run "
-		"sudo on %s.\n"), user_name, user_srunhost);
+		"sudo on %s.\n"), ctx->user.name, ctx->runas.shost);
 	} else {
+	    const struct passwd *runas_pw =
+		ctx->runas.list_pw ? ctx->runas.list_pw : ctx->runas.pw;
+	    const char *cmnd1 = ctx->user.cmnd;
+	    const char *cmnd2 = "";
+
+	    if (ISSET(ctx->mode, MODE_CHECK)) {
+		/* For "sudo -l command" the command run is in runas.argv[1]. */
+		cmnd1 = "list ";
+		cmnd2 = ctx->runas.argv[1];
+	    }
 	    sudo_printf(SUDO_CONV_ERROR_MSG, _("Sorry, user %s is not allowed "
 		"to execute '%s%s%s%s' as %s%s%s on %s.\n"),
-		user_name, user_cmnd, list_cmnd ? list_cmnd : "",
-		user_args ? " " : "", user_args ? user_args : "",
-		list_pw ? list_pw->pw_name : runas_pw ?
-		runas_pw->pw_name : user_name, runas_gr ? ":" : "",
-		runas_gr ? runas_gr->gr_name : "", user_host);
+		ctx->user.name, cmnd1, cmnd2, ctx->user.cmnd_args ? " " : "",
+		ctx->user.cmnd_args ? ctx->user.cmnd_args : "",
+		runas_pw ? runas_pw->pw_name : ctx->user.name,
+		ctx->runas.gr ? ":" : "",
+		ctx->runas.gr ? ctx->runas.gr->gr_name : "",
+		ctx->user.host);
+	    if (def_cmddenial_message != NULL) {
+		sudo_printf(SUDO_CONV_ERROR_MSG, "%s\n", def_cmddenial_message);
+	    }
 	}
 	if (mailit) {
 	    sudo_printf(SUDO_CONV_ERROR_MSG, "%s",
@@ -342,21 +366,23 @@ log_denial(int status, bool inform_user)
  * Log and audit that user was not allowed to run the command.
  */
 bool
-log_failure(int status, int flags)
+log_failure(const struct sudoers_context *ctx, unsigned int status,
+    int cmnd_status)
 {
     bool ret, inform_user = true;
     debug_decl(log_failure, SUDOERS_DEBUG_LOGGING);
 
     /* The user doesn't always get to see the log message (path info). */
-    if (!ISSET(status, FLAG_NO_USER | FLAG_NO_HOST) && list_pw == NULL &&
-	    def_path_info && (flags == NOT_FOUND_DOT || flags == NOT_FOUND))
+    if (!ISSET(status, FLAG_NO_USER | FLAG_NO_HOST) &&
+	    ctx->runas.list_pw == NULL && def_path_info &&
+	    (cmnd_status == NOT_FOUND_DOT || cmnd_status == NOT_FOUND))
 	inform_user = false;
-    ret = log_denial(status, inform_user);
+    ret = log_denial(ctx, status, inform_user);
 
     if (!inform_user) {
-	const char *cmnd = user_cmnd;
-	if (ISSET(sudo_mode, MODE_CHECK))
-	    cmnd = list_cmnd ? list_cmnd : NewArgv[1];
+	const char *cmnd = ctx->user.cmnd;
+	if (ISSET(ctx->mode, MODE_CHECK))
+	    cmnd = ctx->user.cmnd_list ? ctx->user.cmnd_list : ctx->runas.argv[1];
 
 	/*
 	 * We'd like to not leak path info at all here, but that can
@@ -365,9 +391,9 @@ log_failure(int status, int flags)
 	 * is just "no foo in path" since the user can trivially set
 	 * their path to just contain a single dir.
 	 */
-	if (flags == NOT_FOUND)
+	if (cmnd_status == NOT_FOUND)
 	    sudo_warnx(U_("%s: command not found"), cmnd);
-	else if (flags == NOT_FOUND_DOT)
+	else if (cmnd_status == NOT_FOUND_DOT)
 	    sudo_warnx(U_("ignoring \"%s\" found in '.'\nUse \"sudo ./%s\" if this is the \"%s\" you wish to run."), cmnd, cmnd, cmnd);
     }
 
@@ -381,7 +407,7 @@ log_failure(int status, int flags)
 static char *
 fmt_authfail_message(unsigned int tries)
 {
-    char numbuf[(((sizeof(int) * 8) + 2) / 3) + 2];
+    char numbuf[STRLEN_MAX_UNSIGNED(unsigned int) + 1];
     char *dst, *dst_end, *ret = NULL;
     const char *src;
     size_t len;
@@ -394,7 +420,7 @@ fmt_authfail_message(unsigned int tries)
 	debug_return_ptr(ret);
     }
 
-    len = snprintf(numbuf, sizeof(numbuf), "%u", tries);
+    len = (size_t)snprintf(numbuf, sizeof(numbuf), "%u", tries);
     if (len >= sizeof(numbuf))
 	goto overflow;
 
@@ -436,7 +462,7 @@ fmt_authfail_message(unsigned int tries)
 		src++;
 		break;
 	    case 'd':
-		len = strlcpy(dst, numbuf, dst_end - dst);
+		len = strlcpy(dst, numbuf, (size_t)(dst_end - dst));
 		if (len >= (size_t)(dst_end - dst))
 		    goto overflow;
 		dst += len;
@@ -468,7 +494,8 @@ overflow:
  * Log and audit that user was not able to authenticate themselves.
  */
 bool
-log_auth_failure(int status, unsigned int tries)
+log_auth_failure(const struct sudoers_context *ctx, unsigned int status,
+    unsigned int tries)
 {
     char *message = NULL;
     int oldlocale;
@@ -478,11 +505,16 @@ log_auth_failure(int status, unsigned int tries)
     debug_decl(log_auth_failure, SUDOERS_DEBUG_LOGGING);
 
     /* Do auditing first (audit_failure() handles the locale itself). */
-    audit_failure(NewArgv, "%s", N_("authentication failure"));
+    audit_failure(ctx, ctx->runas.argv, "%s", N_("authentication failure"));
 
-    /* If sudoers denied the command we'll log that separately. */
-    if (!ISSET(status, FLAG_BAD_PASSWORD|FLAG_NO_USER_INPUT))
+    if (ISSET(status, FLAG_NO_USER_INPUT)) {
+	/* For "sudo -n", only log the entry if an actual command was run. */
+	if (ISSET(ctx->mode, MODE_LIST|MODE_VALIDATE))
+	    logit = false;
+    } else if (!ISSET(status, FLAG_BAD_PASSWORD)) {
+	/* Authenticated OK, sudoers denials are logged separately. */
 	logit = false;
+    }
 
     /*
      * Do we need to send mail?
@@ -498,7 +530,7 @@ log_auth_failure(int status, unsigned int tries)
 	    logit = false;
     } else {
 	/* Command denied, auth failed; make sure we don't send mail twice. */
-	if (def_mail_badpass && !should_mail(status))
+	if (def_mail_badpass && !should_mail(ctx, status))
 	    mailit = true;
 	/* Don't log the bad password message, we'll log a denial instead. */
 	logit = false;
@@ -513,11 +545,11 @@ log_auth_failure(int status, unsigned int tries)
 	    if (message == NULL) {
 		ret = false;
 	    } else {
-		ret = log_reject(message, logit, mailit);
+		ret = log_reject(ctx, message, logit, mailit);
 		free(message);
 	    }
 	} else {
-	    ret = log_reject(_("a password is required"), logit, mailit);
+	    ret = log_reject(ctx, _("a password is required"), logit, mailit);
 	}
 
 	/* Restore locale. */
@@ -548,7 +580,7 @@ log_auth_failure(int status, unsigned int tries)
  * Log and potentially mail the allowed command.
  */
 bool
-log_allowed(struct eventlog *evlog)
+log_allowed(const struct sudoers_context *ctx, struct eventlog *evlog)
 {
     int oldlocale;
     int evl_flags = 0;
@@ -556,7 +588,7 @@ log_allowed(struct eventlog *evlog)
     debug_decl(log_allowed, SUDOERS_DEBUG_LOGGING);
 
     /* Send mail based on status. */
-    mailit = should_mail(VALIDATE_SUCCESS);
+    mailit = should_mail(ctx, VALIDATE_SUCCESS);
 
     if (def_log_allowed || mailit) {
 	/* Log and mail messages should be in the sudoers locale. */
@@ -577,7 +609,7 @@ log_allowed(struct eventlog *evlog)
 }
 
 bool
-log_exit_status(int status)
+log_exit_status(const struct sudoers_context *ctx, int status)
 {
     struct eventlog evlog;
     int evl_flags = 0;
@@ -591,12 +623,12 @@ log_exit_status(int status)
     debug_decl(log_exit_status, SUDOERS_DEBUG_LOGGING);
 
     if (def_log_exit_status || def_mail_always) {
-	if (sudo_gettime_real(&run_time) == -1) {
+	if (sudo_gettime_awake(&run_time) == -1) {
 	    sudo_warn("%s", U_("unable to get time of day"));
 	    ret = false;
 	    goto done;
 	}
-	sudo_timespecsub(&run_time, &sudo_user.submit_time, &run_time);
+	sudo_timespecsub(&run_time, &ctx->start_time, &run_time);
 
         if (WIFEXITED(status)) {
 	    exit_value = WEXITSTATUS(status);
@@ -616,8 +648,8 @@ log_exit_status(int status)
 	/* Log and mail messages should be in the sudoers locale. */
 	sudoers_setlocale(SUDOERS_LOCALE_SUDOERS, &oldlocale);
 
-	sudoers_to_eventlog(&evlog, saved_cmnd, saved_argv, env_get(),
-	    sudo_user.uuid_str);
+	sudoers_to_eventlog(ctx, &evlog, ctx->runas.cmnd_saved,
+	    ctx->runas.argv_saved, NULL, ctx->uuid_str);
 	if (def_mail_always) {
 	    SET(evl_flags, EVLOG_MAIL);
 	    if (!def_log_exit_status)
@@ -640,6 +672,7 @@ done:
 /*
  * Add message to the parse error journal, which takes ownership of it.
  * The message will be freed once the journal is processed.
+ * Returns true if message was journaled (and consumed), else false.
  */
 static bool
 journal_parse_error(char *message)
@@ -652,17 +685,17 @@ journal_parse_error(char *message)
 	debug_return_bool(false);
     pe->errstr = message;
     STAILQ_INSERT_TAIL(&parse_error_list, pe, entries);
-    debug_return_bool(false);
+    debug_return_bool(true);
 }
 
 /*
  * Perform logging for log_warning()/log_warningx().
  */
 static bool
-vlog_warning(int flags, int errnum, const char *fmt, va_list ap)
+vlog_warning(const struct sudoers_context *ctx, unsigned int flags,
+    int errnum, const char * restrict fmt, va_list ap)
 {
     struct eventlog evlog;
-    struct timespec now;
     const char *errstr = NULL;
     char *message;
     bool ret = true;
@@ -674,7 +707,7 @@ vlog_warning(int flags, int errnum, const char *fmt, va_list ap)
     /* Do auditing first (audit_failure() handles the locale itself). */
     if (ISSET(flags, SLOG_AUDIT)) {
 	va_copy(ap2, ap);
-	vaudit_failure(NewArgv, fmt, ap2);
+	vaudit_failure(ctx, ctx->runas.argv, fmt, ap2);
 	va_end(ap2);
     }
 
@@ -707,10 +740,6 @@ vlog_warning(int flags, int errnum, const char *fmt, va_list ap)
     }
 
     if (ISSET(flags, SLOG_SEND_MAIL) || !ISSET(flags, SLOG_NO_LOG)) {
-	if (sudo_gettime_real(&now) == -1) {
-	    sudo_warn("%s", U_("unable to get time of day"));
-	    goto done;
-	}
 	if (ISSET(flags, SLOG_RAW_MSG))
 	    SET(evl_flags, EVLOG_RAW);
 	if (ISSET(flags, SLOG_SEND_MAIL)) {
@@ -718,11 +747,11 @@ vlog_warning(int flags, int errnum, const char *fmt, va_list ap)
 	    if (ISSET(flags, SLOG_NO_LOG))
 		SET(evl_flags, EVLOG_MAIL_ONLY);
 	}
-	sudoers_to_eventlog(&evlog, safe_cmnd, NewArgv, env_get(),
-	    sudo_user.uuid_str);
-	if (!eventlog_alert(&evlog, evl_flags, &now, message, errstr))
+	sudoers_to_eventlog(ctx, &evlog, ctx->runas.cmnd, ctx->runas.argv,
+	    NULL, ctx->uuid_str);
+	if (!eventlog_alert(&evlog, evl_flags, &evlog.event_time, message, errstr))
 	    ret = false;
-	if (!log_server_alert(&evlog, &now, message, errstr))
+	if (!log_server_alert(ctx, &evlog, message, errstr))
 	    ret = false;
     }
 
@@ -737,9 +766,11 @@ vlog_warning(int flags, int errnum, const char *fmt, va_list ap)
 	    copy = strdup(message);
 	}
 	if (copy != NULL) {
-	    /* journal_parse_error() takes ownership of copy. */
-	    if (!journal_parse_error(copy))
+	    /* journal_parse_error() takes ownership of copy on success. */
+	    if (!journal_parse_error(copy)) {
+		free(copy);
 		ret = false;
+	    }
 	}
     }
 
@@ -766,7 +797,8 @@ done:
 }
 
 bool
-log_warning(int flags, const char *fmt, ...)
+log_warning(const struct sudoers_context *ctx, unsigned int flags,
+    const char * restrict fmt, ...)
 {
     va_list ap;
     bool ret;
@@ -774,14 +806,15 @@ log_warning(int flags, const char *fmt, ...)
 
     /* Log the error. */
     va_start(ap, fmt);
-    ret = vlog_warning(flags|SLOG_USE_ERRNO, errno, fmt, ap);
+    ret = vlog_warning(ctx, flags|SLOG_USE_ERRNO, errno, fmt, ap);
     va_end(ap);
 
     debug_return_bool(ret);
 }
 
 bool
-log_warningx(int flags, const char *fmt, ...)
+log_warningx(const struct sudoers_context *ctx, unsigned int flags,
+    const char * restrict fmt, ...)
 {
     va_list ap;
     bool ret;
@@ -789,14 +822,15 @@ log_warningx(int flags, const char *fmt, ...)
 
     /* Log the error. */
     va_start(ap, fmt);
-    ret = vlog_warning(flags, 0, fmt, ap);
+    ret = vlog_warning(ctx, flags, 0, fmt, ap);
     va_end(ap);
 
     debug_return_bool(ret);
 }
 
 bool
-gai_log_warning(int flags, int errnum, const char *fmt, ...)
+gai_log_warning(const struct sudoers_context *ctx, unsigned int flags,
+    int errnum, const char * restrict fmt, ...)
 {
     va_list ap;
     bool ret;
@@ -804,7 +838,7 @@ gai_log_warning(int flags, int errnum, const char *fmt, ...)
 
     /* Log the error. */
     va_start(ap, fmt);
-    ret = vlog_warning(flags|SLOG_GAI_ERRNO, errnum, fmt, ap);
+    ret = vlog_warning(ctx, flags|SLOG_GAI_ERRNO, errnum, fmt, ap);
     va_end(ap);
 
     debug_return_bool(ret);
@@ -815,7 +849,7 @@ gai_log_warning(int flags, int errnum, const char *fmt, ...)
  * Frees the list of parse errors when done.
  */
 bool
-mail_parse_errors(void)
+mail_parse_errors(const struct sudoers_context *ctx)
 {
     const int evl_flags = EVLOG_RAW;
     struct parse_error *pe;
@@ -833,8 +867,8 @@ mail_parse_errors(void)
 	sudo_warn("%s", U_("unable to get time of day"));
 	goto done;
     }
-    sudoers_to_eventlog(&evlog, safe_cmnd, NewArgv, env_get(),
-	sudo_user.uuid_str);
+    sudoers_to_eventlog(ctx, &evlog, ctx->runas.cmnd, ctx->runas.argv,
+	NULL, ctx->uuid_str);
 
     /* Convert parse_error_list to a string vector. */
     n = 0;
@@ -871,10 +905,10 @@ done:
  * Does not write the message to stderr.
  */
 bool
-log_parse_error(const char *file, int line, int column, const char *fmt,
-    va_list args)
+log_parse_error(const struct sudoers_context *ctx, const char *file,
+    int line, int column, const char * restrict fmt, va_list args)
 {
-    const int flags = SLOG_RAW_MSG|SLOG_NO_STDERR;
+    const unsigned int flags = SLOG_RAW_MSG|SLOG_NO_STDERR;
     char *message, *tofree = NULL;
     const char *errstr;
     bool ret;
@@ -893,10 +927,10 @@ log_parse_error(const char *file, int line, int column, const char *fmt,
     }
 
     if (line > 0) {
-	ret = log_warningx(flags, N_("%s:%d:%d: %s"), file, line, column,
+	ret = log_warningx(ctx, flags, N_("%s:%d:%d: %s"), file, line, column,
 	    errstr);
     } else {
-	ret = log_warningx(flags, N_("%s: %s"), file, errstr);
+	ret = log_warningx(ctx, flags, N_("%s: %s"), file, errstr);
     }
 
     /* Journal parse error for later mailing. */
@@ -906,7 +940,10 @@ log_parse_error(const char *file, int line, int column, const char *fmt,
 	len = asprintf(&message, _("%s: %s"), file, errstr);
     }
     if (len != -1) {
-	journal_parse_error(message);
+	if (!journal_parse_error(message)) {
+	    free(message);
+	    ret = false;
+	}
     } else {
 	ret = false;
     }
@@ -920,7 +957,7 @@ log_parse_error(const char *file, int line, int column, const char *fmt,
  * Determine whether we should send mail based on "status" and defaults options.
  */
 static bool
-should_mail(int status)
+should_mail(const struct sudoers_context *ctx, unsigned int status)
 {
     debug_decl(should_mail, SUDOERS_DEBUG_LOGGING);
 
@@ -928,7 +965,7 @@ should_mail(int status)
 	debug_return_bool(false);
 
     debug_return_bool(def_mail_always || ISSET(status, VALIDATE_ERROR) ||
-	(def_mail_all_cmnds && ISSET(sudo_mode, (MODE_RUN|MODE_EDIT))) ||
+	(def_mail_all_cmnds && ISSET(ctx->mode, (MODE_RUN|MODE_EDIT))) ||
 	(def_mail_no_user && ISSET(status, FLAG_NO_USER)) ||
 	(def_mail_no_host && ISSET(status, FLAG_NO_HOST)) ||
 	(def_mail_no_perms && !ISSET(status, VALIDATE_SUCCESS)));
@@ -939,51 +976,56 @@ should_mail(int status)
  * The values in the resulting eventlog struct should not be freed.
  */
 void
-sudoers_to_eventlog(struct eventlog *evlog, const char *cmnd,
-    char * const argv[], char * const envp[], const char *uuid_str)
+sudoers_to_eventlog(const struct sudoers_context *ctx, struct eventlog *evlog,
+    const char *cmnd, char * const runargv[], char * const runenv[],
+    const char *uuid_str)
 {
     struct group *grp;
     debug_decl(sudoers_to_eventlog, SUDOERS_DEBUG_LOGGING);
 
     /* We rely on the reference held by the group cache. */
-    if ((grp = sudo_getgrgid(sudo_user.pw->pw_gid)) != NULL)
+    if ((grp = sudo_getgrgid(ctx->user.pw->pw_gid)) != NULL)
 	sudo_gr_delref(grp);
 
     memset(evlog, 0, sizeof(*evlog));
-    evlog->iolog_file = sudo_user.iolog_file;
-    evlog->iolog_path = sudo_user.iolog_path;
-    evlog->command = cmnd ? (char *)cmnd : (argv ? argv[0] : NULL);
-    evlog->cwd = user_cwd;
+    evlog->iolog_file = ctx->iolog_file;
+    evlog->iolog_path = ctx->iolog_path;
+    evlog->command = cmnd ? (char *)cmnd : (runargv ? runargv[0] : NULL);
+    evlog->cwd = ctx->user.cwd;
     if (def_runchroot != NULL && strcmp(def_runchroot, "*") != 0) {
 	evlog->runchroot = def_runchroot;
     }
     if (def_runcwd && strcmp(def_runcwd, "*") != 0) {
 	evlog->runcwd = def_runcwd;
-    } else if (ISSET(sudo_mode, MODE_LOGIN_SHELL) && runas_pw != NULL) {
-	evlog->runcwd = runas_pw->pw_dir;
+    } else if (ISSET(ctx->mode, MODE_LOGIN_SHELL) && ctx->runas.pw != NULL) {
+	evlog->runcwd = ctx->runas.pw->pw_dir;
     } else {
-	evlog->runcwd = user_cwd;
+	evlog->runcwd = ctx->user.cwd;
     }
-    evlog->rungroup = runas_gr ? runas_gr->gr_name : sudo_user.runas_group;
-    evlog->submithost = user_host;
-    evlog->submituser = user_name;
+    evlog->rungroup = ctx->runas.gr ? ctx->runas.gr->gr_name : ctx->runas.group;
+    evlog->source = ctx->source;
+    evlog->submithost = ctx->user.host;
+    evlog->submituser = ctx->user.name;
     if (grp != NULL)
 	evlog->submitgroup = grp->gr_name;
-    evlog->ttyname = user_ttypath;
-    evlog->argv = (char **)argv;
-    evlog->env_add = (char **)sudo_user.env_vars;
-    evlog->envp = (char **)envp;
-    evlog->submit_time = sudo_user.submit_time;
-    evlog->lines = sudo_user.lines;
-    evlog->columns = sudo_user.cols;
-    if (runas_pw != NULL) {
-	evlog->rungid = runas_pw->pw_gid;
-	evlog->runuid = runas_pw->pw_uid;
-	evlog->runuser = runas_pw->pw_name;
+    evlog->ttyname = ctx->user.ttypath;
+    evlog->runargv = (char **)runargv;
+    evlog->env_add = (char **)ctx->user.env_add;
+    evlog->runenv = (char **)runenv;
+    evlog->submitenv = (char **)ctx->user.envp;
+    if (sudo_gettime_real(&evlog->event_time) == -1) {
+	sudo_warn("%s", U_("unable to get time of day"));
+    }
+    evlog->lines = ctx->user.lines;
+    evlog->columns = ctx->user.cols;
+    if (ctx->runas.pw != NULL) {
+	evlog->rungid = ctx->runas.pw->pw_gid;
+	evlog->runuid = ctx->runas.pw->pw_uid;
+	evlog->runuser = ctx->runas.pw->pw_name;
     } else {
 	evlog->rungid = (gid_t)-1;
 	evlog->runuid = (uid_t)-1;
-	evlog->runuser = sudo_user.runas_user;
+	evlog->runuser = ctx->runas.user;
     }
     if (uuid_str == NULL) {
 	unsigned char uuid[16];
@@ -994,12 +1036,12 @@ sudoers_to_eventlog(struct eventlog *evlog, const char *cmnd,
     } else {
 	strlcpy(evlog->uuid_str, uuid_str, sizeof(evlog->uuid_str));
     }
-    if (ISSET(sudo_mode, MODE_POLICY_INTERCEPTED)) {
-	struct timespec now;
-	if (sudo_gettime_real(&now) == -1) {
+    if (ISSET(ctx->mode, MODE_POLICY_INTERCEPTED)) {
+	if (sudo_gettime_awake(&evlog->iolog_offset) == -1) {
 	    sudo_warn("%s", U_("unable to get time of day"));
 	} else {
-	    sudo_timespecsub(&now, &sudo_user.submit_time, &evlog->iolog_offset);
+	    sudo_timespecsub(&evlog->iolog_offset, &ctx->start_time,
+		&evlog->iolog_offset);
 	}
     }
 
@@ -1022,7 +1064,7 @@ sudoers_log_open(int type, const char *log_file)
 	    break;
 	case EVLOG_FILE:
 	    /* Open log file as root, mode 0600 (cannot append to JSON). */
-	    if (def_log_format == json) {
+	    if (def_log_format == json || def_log_format == json_pretty) {
 		flags = O_RDWR|O_CREAT;
 		omode = "w";
 	    } else {
@@ -1030,7 +1072,7 @@ sudoers_log_open(int type, const char *log_file)
 		omode = "a";
 	    }
 	    oldmask = umask(S_IRWXG|S_IRWXO);
-	    uid_changed = set_perms(PERM_ROOT);
+	    uid_changed = set_perms(NULL, PERM_ROOT);
 	    fd = open(log_file, flags, S_IRUSR|S_IWUSR);
 	    if (uid_changed && !restore_perms()) {
 		if (fd != -1) {
@@ -1042,8 +1084,7 @@ sudoers_log_open(int type, const char *log_file)
 	    if (fd == -1 || (fp = fdopen(fd, omode)) == NULL) {
 		if (!warned) {
 		    warned = true;
-		    log_warning(SLOG_SEND_MAIL|SLOG_NO_LOG,
-			N_("unable to open log file %s"), log_file);
+		    sudo_warn(U_("unable to open log file %s"), log_file);
 		}
 		if (fd != -1)
 		    close(fd);
@@ -1075,8 +1116,7 @@ sudoers_log_close(int type, FILE *fp)
 	    (void)fflush(fp);
 	    if (ferror(fp) && !warned) {
 		warned = true;
-		log_warning(SLOG_SEND_MAIL|SLOG_NO_LOG,
-		    N_("unable to write log file: %s"), def_logfile);
+		sudo_warn(U_("unable to write log file %s"), def_logfile);
 	    }
 	    fclose(fp);
 	    break;
@@ -1100,8 +1140,8 @@ init_eventlog_config(void)
     if (def_logfile)
 	logtype |= EVLOG_FILE;
 
+    sudoers_set_log_format(def_log_format);
     eventlog_set_type(logtype);
-    eventlog_set_format(def_log_format == sudo ? EVLOG_SUDO : EVLOG_JSON);
     eventlog_set_syslog_acceptpri(def_syslog_goodpri);
     eventlog_set_syslog_rejectpri(def_syslog_badpri);
     eventlog_set_syslog_alertpri(def_syslog_badpri);
