@@ -25,10 +25,10 @@
 #include <time.h>
 #include <unistd.h>
 
-#include "sudo_compat.h"
-#include "sudo_util.h"
-#include "sudo_fatal.h"
-#include "check.h"
+#include <sudo_compat.h>
+#include <sudo_util.h>
+#include <sudo_fatal.h>
+#include <timestamp.h>
 
 sudo_dso_public int main(int argc, char *argv[]);
 
@@ -80,15 +80,28 @@ get_now(struct timespec *now)
 int
 main(int argc, char *argv[])
 {
-    int ntests = 0, errors = 0;
+    int ch, ntests = 0, errors = 0;
     struct timespec now, then, delta;
     time_t timeoff = 0;
     pid_t pids[2];
     char *faketime;
-    int i;
+    unsigned int i;
 
     initprogname(argc > 0 ? argv[0] : "check_starttime");
 
+    while ((ch = getopt(argc, argv, "v")) != -1) {
+	switch (ch) {
+	case 'v':
+	    /* ignored */
+	    break;
+	default:
+	    fprintf(stderr, "usage: %s [-v]\n", getprogname());
+	    return EXIT_FAILURE;
+	}
+    }
+    argc -= optind;
+    argv += optind;
+
     if (get_now(&now) == -1)
 	sudo_fatal_nodebug("unable to get current time");
 
