@@ -100,7 +100,7 @@ int sudo_ldap_innetgr_int(void *v, const char *netgr, const char *host, const ch
 /* ldap_util.c */
 bool sudo_ldap_is_negated(char **valp);
 size_t sudo_ldap_value_len(const char *value);
-size_t sudo_ldap_value_cat(char *dst, const char *src, size_t size);
+size_t sudo_ldap_value_cat(char * restrict dst, const char * restrict src, size_t size);
 char *sudo_ldap_value_dup(const char *src);
 int sudo_ldap_parse_option(char *optstr, char **varp, char **valp);
 struct privilege *sudo_ldap_role_to_priv(const char *cn, void *hosts, void *runasusers, void *runasgroups, void *cmnds, void *opts, const char *notbefore, const char *notafter, bool warnings, bool store_options, sudo_ldap_iter_t iter);
