@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 1993-1996, 1998-2005, 2007-2023
+ * Copyright (c) 1993-1996, 1998-2005, 2007-2024
  *	Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -24,7 +24,7 @@
 #ifndef SUDOERS_SUDOERS_H
 #define SUDOERS_SUDOERS_H
 
-#include <sys/types.h>		/* for gid_t, mode_t, pid_t, size_t, uid_t */
+#include <sys/types.h>		/* dev_t, gid_t, mode_t, pid_t, size_t, uid_t */
 #include <limits.h>
 #ifdef HAVE_STDBOOL_H
 # include <stdbool.h>
@@ -90,14 +90,14 @@ struct sudoers_parser_config {
     gid_t sudoers_gid;
 };
 #define SUDOERS_PARSER_CONFIG_INITIALIZER {				\
-    NULL,	/* sudoers_path */					\
-    false,	/* strict */						\
-    1,		/* verbose level 1 */					\
-    true,	/* recovery */						\
-    false,	/* ignore_perms */					\
-    SUDOERS_MODE,							\
-    SUDOERS_UID,							\
-    SUDOERS_GID								\
+    .sudoers_path = NULL,						\
+    .strict = false,							\
+    .verbose = 1,							\
+    .recovery = true,							\
+    .ignore_perms = false,						\
+    .sudoers_mode = SUDOERS_MODE,					\
+    .sudoers_uid = SUDOERS_UID,						\
+    .sudoers_gid = SUDOERS_GID						\
 }
 
 /*
@@ -109,6 +109,11 @@ struct sudoers_plugin_settings {
     const char *ldap_secret;
     unsigned int flags;
 };
+#define SUDOERS_PLUGIN_SETTINGS_INITIALIZER {				\
+    .plugin_dir = _PATH_SUDO_PLUGIN_DIR,				\
+    .ldap_conf = _PATH_LDAP_CONF,					\
+    .ldap_secret = _PATH_LDAP_SECRET					\
+}
 
 /*
  * Info pertaining to the invoking user.
@@ -137,6 +142,7 @@ struct sudoers_user_context {
     int   lines;
     int   cols;
     int   timeout;
+    dev_t ttydev;
     mode_t umask;
     uid_t euid;
     uid_t uid;
@@ -168,24 +174,13 @@ struct sudoers_runas_context {
     char *host;
     char *shost;
     char *user;
-#ifdef HAVE_SELINUX
     char *role;
     char *type;
-#endif
-#ifdef HAVE_APPARMOR
     char *apparmor_profile;
-#endif
-#ifdef HAVE_PRIV_SET
     char *privs;
     char *limitprivs;
-#endif
 };
 
-#define SUDOERS_CONTEXT_INITIALIZER {					\
-    SUDOERS_PARSER_CONFIG_INITIALIZER,					\
-    { _PATH_LDAP_CONF, _PATH_LDAP_SECRET, _PATH_SUDO_PLUGIN_DIR }	\
-}
-
 /*
  * Global configuration for the sudoers module.
  */
@@ -194,7 +189,7 @@ struct sudoers_context {
     struct sudoers_plugin_settings settings;
     struct sudoers_user_context user;
     struct sudoers_runas_context runas;
-    struct timespec submit_time;
+    struct timespec start_time;
     char *source;
     char *iolog_file;
     char *iolog_dir;
@@ -203,6 +198,10 @@ struct sudoers_context {
     unsigned int mode;
     char uuid_str[37];
 };
+#define SUDOERS_CONTEXT_INITIALIZER {					\
+    SUDOERS_PARSER_CONFIG_INITIALIZER,					\
+    SUDOERS_PLUGIN_SETTINGS_INITIALIZER,				\
+}
 
 /*
  * sudo_get_gidlist() type values
@@ -310,7 +309,6 @@ struct sudoers_context {
 struct sudo_lbuf;
 struct passwd;
 struct stat;
-struct timespec;
 
 /*
  * Function prototypes
@@ -455,6 +453,7 @@ void sudoers_cleanup(void);
 bool sudoers_override_umask(void);
 const struct sudoers_context *sudoers_get_context(void);
 bool sudoers_set_mode(unsigned int flags, unsigned int mask);
+bool sudoers_set_log_format(enum def_tuple tuple);
 
 /* sudoers_ctx_free.c */
 void sudoers_ctx_free(struct sudoers_context *ctx);
