@@ -715,9 +715,7 @@ print_aliases(struct sudo_lbuf *lbuf)
 {
     debug_decl(print_aliases, SUDOERS_DEBUG_UTIL);
 
-    alias_apply(&parsed_policy, print_alias, lbuf);
-
-    debug_return_bool(!sudo_lbuf_error(lbuf));
+    debug_return_bool(alias_apply(&parsed_policy, print_alias, lbuf));
 }
 
 static void
