@@ -43,10 +43,10 @@
 # include <sys/task.h>
 #endif
 
-#include "sudo.h"
-#include "sudo_exec.h"
-#include "sudo_plugin.h"
-#include "sudo_plugin_int.h"
+#include <sudo.h>
+#include <sudo_exec.h>
+#include <sudo_plugin.h>
+#include <sudo_plugin_int.h>
 
 #ifdef HAVE_PTRACE_INTERCEPT
 static void
@@ -84,7 +84,7 @@ close_fds(struct command_details *details, int errfd, int intercept_fd)
 }
 
 /*
- * Setup the execution environment immediately prior to the call to execve().
+ * Set up the execution environment immediately prior to the call to execve().
  * Group setup is performed by policy_init_session(), called earlier.
  * Returns true on success and false on failure.
  */
@@ -133,7 +133,7 @@ exec_setup(struct command_details *details, int intercept_fd, int errfd)
 #endif
 #ifdef HAVE_LOGIN_CAP_H
 	if (details->login_class) {
-	    int flags;
+	    unsigned int flags;
 	    login_cap_t *lc;
 
 	    /*
@@ -253,7 +253,7 @@ done:
 }
 
 /*
- * Setup the execution environment and execute the command.
+ * Set up the execution environment and execute the command.
  * If SELinux is enabled, run the command via sesh, otherwise
  * execute it directly.
  * If the exec fails, cstat is filled in with the value of errno.
@@ -376,6 +376,29 @@ sudo_needs_pty(const struct command_details *details)
     return false;
 }
 
+/*
+ * Check whether the specified fd matches the device file that
+ * corresponds to tty_sb.  If tty_sb is NULL, just check whether
+ * fd is a tty.  Always fills in fd_sb (zeroed on error).
+ * Returns true on match, else false.
+ */
+bool
+fd_matches_tty(int fd, struct stat *tty_sb, struct stat *fd_sb)
+{
+    debug_decl(fd_is_user_tty, SUDO_DEBUG_EXEC);
+
+    if (fstat(fd, fd_sb) == -1) {
+	 /* Always initialize fd_sb. */
+	memset(fd_sb, 0, sizeof(*fd_sb));
+	debug_return_bool(false);
+    }
+    if (!S_ISCHR(fd_sb->st_mode))
+	debug_return_bool(false);
+
+    /* Compare with tty_sb if available, else just check that fd is a tty. */
+    debug_return_bool(tty_sb ? tty_sb->st_rdev == fd_sb->st_rdev : isatty(fd));
+}
+
 /*
  * If we are not running the command in a pty, we were not invoked as
  * sudoedit, there is no command timeout and there is no close function,
