@@ -878,11 +878,12 @@ ptrace_write_vec(pid_t pid, struct sudo_ptrace_regs *regs, char **vec,
     unsigned long addr, unsigned long strtab)
 {
     const unsigned long strtab0 = strtab;
+    ssize_t nwritten;
     size_t i;
     debug_decl(ptrace_write_vec, SUDO_DEBUG_EXEC);
 
 #ifdef HAVE_PROCESS_VM_READV
-    ssize_t nwritten = ptrace_writev_vec(pid, regs, vec, addr, strtab);
+    nwritten = ptrace_writev_vec(pid, regs, vec, addr, strtab);
     if (nwritten != -1 || errno != ENOSYS)
 	debug_return_ssize_t(nwritten);
 #endif /* HAVE_PROCESS_VM_READV */
@@ -1385,7 +1386,7 @@ script_matches(const char *script, const char *execpath, int argc,
 	/* Match interpreter. */
 	if (!pathname_matches(execpath, interp, true)) {
 	    /* It is possible for the interpreter to be a script too. */
-	    if (argv > 0 && strcmp(interp, argv[1]) == 0) {
+	    if (argc > 0 && strcmp(interp, argv[1]) == 0) {
 		/* Interpreter args must match for *this* interpreter. */
 		if (interp_args == NULL ||
 			(argc > 1 && strcmp(interp_args, argv[2]) == 0)) {
@@ -1797,7 +1798,7 @@ ptrace_intercept_execve(pid_t pid, struct intercept_closure *closure)
 	goto done;
     }
 
-    /* We can only pass the pathname to exececute via argv[0] (plugin API). */
+    /* We can only pass the pathname to execute via argv[0] (plugin API). */
     orig_argv0 = argv[0] ? argv[0] : (char *)"";
     argv[0] = pathname;
     if (argc == 0) {
