@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 2009-2023 Todd C. Miller <Todd.Miller@sudo.ws>
+ * Copyright (c) 2009-2024 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -43,10 +43,10 @@
 #include <signal.h>
 #include <termios.h>		/* for struct winsize on HP-UX */
 
-#include "sudo.h"
-#include "sudo_exec.h"
-#include "sudo_plugin.h"
-#include "sudo_plugin_int.h"
+#include <sudo.h>
+#include <sudo_exec.h>
+#include <sudo_plugin.h>
+#include <sudo_plugin_int.h>
 
 /* Tail queue of messages to send to the monitor. */
 struct monitor_message {
@@ -56,14 +56,17 @@ struct monitor_message {
 TAILQ_HEAD(monitor_message_list, monitor_message);
 static struct monitor_message_list monitor_messages =
     TAILQ_HEAD_INITIALIZER(monitor_messages);
+static unsigned int term_raw_flags;
+static struct exec_closure pty_ec;
 
 static void sync_ttysize(struct exec_closure *ec);
 static void schedule_signal(struct exec_closure *ec, int signo);
+static void send_command_status(struct exec_closure *ec, int type, int val);
 
 /*
  * Allocate a pty if /dev/tty is a tty.
  * Fills in io_fds[SFD_USERTTY], io_fds[SFD_LEADER] and io_fds[SFD_FOLLOWER].
- * Returns the dyamically allocated pty name on success, NULL on failure.
+ * Returns the dynamically allocated pty name on success, NULL on failure.
  */
 static char *
 pty_setup(struct command_details *details)
@@ -102,33 +105,18 @@ pty_setup(struct command_details *details)
  * Restore user's terminal settings and update utmp, as needed.
  */
 static void
-pty_cleanup_int(struct exec_closure *ec, int wstatus, bool init_only)
+pty_cleanup(struct exec_closure *ec, int wstatus)
 {
-    static struct exec_closure *saved_ec;
     debug_decl(pty_cleanup, SUDO_DEBUG_EXEC);
 
-    /* If initializing, just store a pointer to the closure and return. */
-    if (init_only) {
-	saved_ec = ec;
-	debug_return;
-    }
-
-    /* Use the stored closure if one is not specified. */
-    if (ec == NULL) {
-	if (saved_ec == NULL)
-	    debug_return;
-	ec = saved_ec;
-    }
-
     /* Restore terminal settings. */
     if (ec->term_raw) {
 	/* Only restore the terminal if sudo is the foreground process. */
 	const pid_t tcpgrp = tcgetpgrp(io_fds[SFD_USERTTY]);
 	if (tcpgrp == ec->ppgrp) {
-	    if (sudo_term_restore(io_fds[SFD_USERTTY], false))
-		ec->term_raw = false;
-	    else
+	    if (!sudo_term_restore(io_fds[SFD_USERTTY], false))
 		sudo_warn("%s", U_("unable to restore terminal settings"));
+	    ec->term_raw = false;
 	}
     }
 
@@ -139,30 +127,18 @@ pty_cleanup_int(struct exec_closure *ec, int wstatus, bool init_only)
     debug_return;
 }
 
-static inline void
-pty_cleanup(struct exec_closure *ec, int wstatus)
-{
-    pty_cleanup_int(ec, wstatus, false);
-}
-
-static inline void
-pty_cleanup_init(struct exec_closure *ec)
-{
-    pty_cleanup_int(ec, 0, true);
-}
-
 /*
  * Cleanup hook for sudo_fatal()/sudo_fatalx()
  */
 static void
 pty_cleanup_hook(void)
 {
-    pty_cleanup_int(NULL, 0, false);
+    pty_cleanup(&pty_ec, 0);
 }
 
 /*
- * Check whether we are running in the foregroup.
- * Updates the foreground flag and updates the window size.
+ * Check whether sudo is running in the foreground.
+ * Updates the foreground flag in the closure.
  * Returns 0 if there is no tty, the foreground process group ID
  * on success, or -1 on failure (tty revoked).
  */
@@ -208,8 +184,7 @@ resume_terminal(struct exec_closure *ec)
 
     if (ec->foreground) {
 	/* Foreground process, set tty to raw mode. */
-	if (sudo_term_raw(io_fds[SFD_USERTTY], 0))
-	    ec->term_raw = true;
+	ec->term_raw = sudo_term_raw(io_fds[SFD_USERTTY], term_raw_flags);
     } else {
 	/* Background process, no access to tty. */
 	ec->term_raw = false;
@@ -260,10 +235,10 @@ suspend_sudo_pty(struct exec_closure *ec, int signo)
 	}
 	if (ec->foreground) {
 	    sudo_debug_printf(SUDO_DEBUG_INFO,
-		"%s: command received SIG%s, parent running in the foregound",
+		"%s: command received SIG%s, parent running in the foreground",
 		__func__, signame);
 	    if (!ec->term_raw) {
-		if (sudo_term_raw(io_fds[SFD_USERTTY], 0))
+		if (sudo_term_raw(io_fds[SFD_USERTTY], term_raw_flags))
 		    ec->term_raw = true;
 	    }
 	    ret = SIGCONT_FG; /* resume command in foreground */
@@ -278,10 +253,9 @@ suspend_sudo_pty(struct exec_closure *ec, int signo)
 
 	/* Restore original tty mode before suspending. */
 	if (ec->term_raw) {
-	    if (sudo_term_restore(io_fds[SFD_USERTTY], false))
-		ec->term_raw = false;
-	    else
+	    if (!sudo_term_restore(io_fds[SFD_USERTTY], false))
 		sudo_warn("%s", U_("unable to restore terminal settings"));
+	    ec->term_raw = false;
 	}
 
 	/* Log the suspend event. */
@@ -358,6 +332,32 @@ sigttin(int signo)
     got_sigttin = 1;
 }
 
+/*
+ * Close the leader fd to revoke the pty and signal the foreground pgrp.
+ * We send SIGHUP to the foreground process group (or the command's
+ * process group if no pty) after closing the leader fd.  We cannot
+ * just forward the SIGHUP we receive from the kernel since the
+ * command may not be the foreground process.  This fixes a problem
+ * on Linux with, e.g. "sudo su" where su(1) blocks SIGHUP.
+ */
+static void
+revoke_pty(struct exec_closure *ec)
+{
+    pid_t pgrp = ec->cmnd_pid;
+    debug_decl(revoke_pty, SUDO_DEBUG_EXEC);
+
+    sudo_debug_printf(SUDO_DEBUG_NOTICE, "user's tty revoked");
+    if (io_fds[SFD_LEADER] != -1) {
+	const pid_t tcpgrp = tcgetpgrp(io_fds[SFD_LEADER]);
+	if (tcpgrp != -1)
+	    pgrp = tcpgrp;
+	close(io_fds[SFD_LEADER]);
+    }
+    sudo_debug_printf(SUDO_DEBUG_NOTICE, "%s: killpg(%d, SIGHUP)",
+	__func__, (int)pgrp);
+    kill(pgrp, SIGHUP);
+}
+
 /*
  * Read an iobuf that is ready.
  */
@@ -410,19 +410,28 @@ read_callback(int fd, int what, void *v)
 	    ev_free_by_fd(evbase, fd);
 	    /* If writer already consumed the buffer, close it too. */
 	    if (iob->wevent != NULL && iob->off == iob->len) {
-		safe_close(sudo_ev_get_fd(iob->wevent));
-		ev_free_by_fd(evbase, sudo_ev_get_fd(iob->wevent));
+		/*
+		 * Don't close the pty leader yet, it will invalidate the pty.
+		 * We ask the monitor to signal the running process first.
+		 */
+		const int wfd = sudo_ev_get_fd(iob->wevent);
+		if (wfd == io_fds[SFD_LEADER]) {
+		    revoke_pty(iob->ec);
+		} else {
+		    safe_close(wfd);
+		}
+		ev_free_by_fd(evbase, wfd);
 		iob->off = iob->len = 0;
 	    }
 	    break;
 	default:
 	    sudo_debug_printf(SUDO_DEBUG_INFO,
 		"read %zd bytes from fd %d", n, fd);
-	    if (!iob->action(iob->buf + iob->len, n, iob)) {
+	    if (!iob->action(iob->buf + iob->len, (unsigned int)n, iob)) {
 		terminate_command(iob->ec->cmnd_pid, true);
 		iob->ec->cmnd_pid = -1;
 	    }
-	    iob->len += n;
+	    iob->len += (unsigned int)n;
 	    /* Disable reader if buffer is full. */
 	    if (iob->len == sizeof(iob->buf))
 		sudo_ev_del(evbase, iob->revent);
@@ -484,12 +493,21 @@ write_callback(int fd, int what, void *v)
 	case EBADF:
 	    /* other end of pipe closed or pty revoked */
 	    sudo_debug_printf(SUDO_DEBUG_INFO,
-		"unable to write %d bytes to fd %d",
+		"unable to write %u bytes to fd %d",
 		iob->len - iob->off, fd);
 	    /* Close reader if there is one. */
 	    if (iob->revent != NULL) {
-		safe_close(sudo_ev_get_fd(iob->revent));
-		ev_free_by_fd(evbase, sudo_ev_get_fd(iob->revent));
+		/*
+		 * Don't close the pty leader, it will invalidate the pty.
+		 * We ask the monitor to signal the running process first.
+		 */
+		const int rfd = sudo_ev_get_fd(iob->revent);
+		if (rfd == io_fds[SFD_LEADER]) {
+		    revoke_pty(iob->ec);
+		} else {
+		    safe_close(rfd);
+		}
+		ev_free_by_fd(evbase, rfd);
 	    }
 	    safe_close(fd);
 	    ev_free_by_fd(evbase, fd);
@@ -514,8 +532,8 @@ write_callback(int fd, int what, void *v)
 	}
     } else {
 	sudo_debug_printf(SUDO_DEBUG_INFO,
-	    "wrote %zd bytes to fd %d", n, fd);
-	iob->off += n;
+	    "wrote %zd of %u bytes to fd %d", n, iob->len - iob->off, fd);
+	iob->off += (unsigned int)n;
 	/* Disable writer and reset the buffer if fully consumed. */
 	if (iob->off == iob->len) {
 	    iob->off = iob->len = 0;
@@ -683,6 +701,13 @@ backchannel_cb(int fd, int what, void *v)
     case sizeof(cstat):
 	/* Check command status. */
 	switch (cstat.type) {
+	case CMD_ERRNO:
+	    /* Monitor was unable to execute command or broken pipe. */
+	    sudo_debug_printf(SUDO_DEBUG_INFO, "errno from monitor: %s",
+		strerror(cstat.val));
+	    sudo_ev_loopbreak(ec->evbase);
+	    *ec->cstat = cstat;
+	    break;
 	case CMD_PID:
 	    ec->cmnd_pid = cstat.val;
 	    sudo_debug_printf(SUDO_DEBUG_INFO, "executed %s, pid %d",
@@ -720,13 +745,6 @@ backchannel_cb(int fd, int what, void *v)
 		*ec->cstat = cstat;
 	    }
 	    break;
-	case CMD_ERRNO:
-	    /* Monitor was unable to execute command or broken pipe. */
-	    sudo_debug_printf(SUDO_DEBUG_INFO, "errno from monitor: %s",
-		strerror(cstat.val));
-	    sudo_ev_loopbreak(ec->evbase);
-	    *ec->cstat = cstat;
-	    break;
 	}
 	/* Keep reading command status messages until EAGAIN or EOF. */
 	break;
@@ -745,7 +763,7 @@ backchannel_cb(int fd, int what, void *v)
 }
 
 /*
- * Handle changes to the monitors's status (SIGCHLD).
+ * Handle changes to the monitor's status (SIGCHLD).
  */
 static void
 handle_sigchld_pty(struct exec_closure *ec)
@@ -846,6 +864,15 @@ signal_cb_pty(int signo, int what, void *v)
     case SIGWINCH:
 	sync_ttysize(ec);
 	break;
+    case SIGHUP:
+	/*
+	 * Avoid forwarding SIGHUP sent by the kernel, it probably means
+	 * that the user's terminal was revoked.  When we detect that the
+	 * terminal has been revoked, the monitor will send SIGHUP itself.
+	 */
+	if (!USER_SIGNALED(sc->siginfo))
+	    break;
+	FALLTHROUGH;
     default:
 	/*
 	 * Do not forward signals sent by the command itself or a member of the
@@ -1068,26 +1095,25 @@ exec_pty(struct command_details *details,
 {
     int io_pipe[3][2] = { { -1, -1 }, { -1, -1 }, { -1, -1 } };
     bool interpose[3] = { false, false, false };
+    struct stat sb, tty_sbuf, *tty_sb = NULL;
     int sv[2], intercept_sv[2] = { -1, -1 };
-    struct exec_closure ec = { 0 };
+    struct exec_closure *ec = &pty_ec;
     struct plugin_container *plugin;
     int evloop_retries = -1;
-    bool pipeline = false;
+    bool cmnd_foreground;
     sigset_t set, oset;
     struct sigaction sa;
-    struct stat sb;
     pid_t ppgrp, sudo_pid;
     debug_decl(exec_pty, SUDO_DEBUG_EXEC);
 
     /*
      * Allocate a pty if sudo is running in a terminal.
      */
-    ec.ptyname = pty_setup(details);
-    if (ec.ptyname == NULL)
+    ec->ptyname = pty_setup(details);
+    if (ec->ptyname == NULL)
 	debug_return_bool(false);
 
     /* Register cleanup function */
-    pty_cleanup_init(&ec);
     sudo_fatal_callback_register(pty_cleanup_hook);
 
     /*
@@ -1159,46 +1185,49 @@ exec_pty(struct command_details *details,
 	/* Read from /dev/tty, write to pty leader */
 	if (!ISSET(details->flags, CD_BACKGROUND)) {
 	    io_buf_new(io_fds[SFD_USERTTY], io_fds[SFD_LEADER],
-		log_ttyin, read_callback, write_callback, &ec);
+		log_ttyin, read_callback, write_callback, ec);
 	}
 
 	/* Read from pty leader, write to /dev/tty */
 	io_buf_new(io_fds[SFD_LEADER], io_fds[SFD_USERTTY],
-	    log_ttyout, read_callback, write_callback, &ec);
+	    log_ttyout, read_callback, write_callback, ec);
 
 	/* Are we the foreground process? */
-	ec.foreground = tcgetpgrp(io_fds[SFD_USERTTY]) == ppgrp;
+	ec->foreground = tcgetpgrp(io_fds[SFD_USERTTY]) == ppgrp;
 	sudo_debug_printf(SUDO_DEBUG_INFO, "sudo is running in the %s",
-	    ec.foreground ? "foreground" : "background");
+	    ec->foreground ? "foreground" : "background");
     }
 
     /*
-     * If stdin, stdout or stderr is not a tty and logging is enabled,
-     * use a pipe to interpose ourselves instead of using the pty fd.
-     * We always use a pipe for stdin when in background mode.
+     * If stdin, stdout or stderr is not the user's tty and logging is
+     * enabled, use a pipe to interpose ourselves instead of using the
+     * pty fd.  We always use a pipe for stdin when in background mode.
      */
-    if (!sudo_isatty(STDIN_FILENO, &sb)) {
+    if (user_details->tty != NULL && stat(user_details->tty, &tty_sbuf) != -1)
+	tty_sb = &tty_sbuf;
+
+    if (!fd_matches_tty(STDIN_FILENO, tty_sb, &sb)) {
 	if (!interpose[STDIN_FILENO]) {
 	    /* Not logging stdin, do not interpose. */
 	    sudo_debug_printf(SUDO_DEBUG_INFO,
-		"stdin not a tty, not logging");
+		"stdin not user's tty, not logging");
 	    if (S_ISFIFO(sb.st_mode))
-		pipeline = true;
+		SET(details->flags, CD_EXEC_BG);
 	    io_fds[SFD_STDIN] = dup(STDIN_FILENO);
 	    if (io_fds[SFD_STDIN] == -1)
 		sudo_fatal("dup");
 	} else {
 	    sudo_debug_printf(SUDO_DEBUG_INFO,
-		"stdin not a tty, creating a pipe");
-	    pipeline = true;
+		"stdin not user's tty, creating a pipe");
+	    SET(details->flags, CD_EXEC_BG);
 	    if (pipe2(io_pipe[STDIN_FILENO], O_CLOEXEC) != 0)
 		sudo_fatal("%s", U_("unable to create pipe"));
 	    io_buf_new(STDIN_FILENO, io_pipe[STDIN_FILENO][1],
-		log_stdin, read_callback, write_callback, &ec);
+		log_stdin, read_callback, write_callback, ec);
 	    io_fds[SFD_STDIN] = io_pipe[STDIN_FILENO][0];
 	}
 
-	if (ec.foreground && ppgrp != sudo_pid) {
+	if (ec->foreground && ppgrp != sudo_pid) {
 	    /*
 	     * If sudo is not the process group leader and stdin is not
 	     * a tty we may be running as a background job via a shell
@@ -1217,51 +1246,52 @@ exec_pty(struct command_details *details,
 	     */
 	    sudo_debug_printf(SUDO_DEBUG_INFO,
 		"terminal input not available, creating empty pipe");
-	    pipeline = true;
+	    SET(details->flags, CD_EXEC_BG);
 	    if (pipe2(io_pipe[STDIN_FILENO], O_CLOEXEC) != 0)
 		sudo_fatal("%s", U_("unable to create pipe"));
 	    io_fds[SFD_STDIN] = io_pipe[STDIN_FILENO][0];
 	    close(io_pipe[STDIN_FILENO][1]);
 	    io_pipe[STDIN_FILENO][1] = -1;
     }
-    if (!sudo_isatty(STDOUT_FILENO, &sb)) {
+    if (!fd_matches_tty(STDOUT_FILENO, tty_sb, &sb)) {
 	if (!interpose[STDOUT_FILENO]) {
 	    /* Not logging stdout, do not interpose. */
 	    sudo_debug_printf(SUDO_DEBUG_INFO,
-		"stdout not a tty, not logging");
-	    if (S_ISFIFO(sb.st_mode))
-		pipeline = true;
+		"stdout not user's tty, not logging");
+	    if (S_ISFIFO(sb.st_mode)) {
+		SET(details->flags, CD_EXEC_BG);
+		term_raw_flags = SUDO_TERM_OFLAG;
+	    }
 	    io_fds[SFD_STDOUT] = dup(STDOUT_FILENO);
 	    if (io_fds[SFD_STDOUT] == -1)
 		sudo_fatal("dup");
 	} else {
 	    sudo_debug_printf(SUDO_DEBUG_INFO,
-		"stdout not a tty, creating a pipe");
-	    pipeline = true;
+		"stdout not user's tty, creating a pipe");
+	    SET(details->flags, CD_EXEC_BG);
+	    term_raw_flags = SUDO_TERM_OFLAG;
 	    if (pipe2(io_pipe[STDOUT_FILENO], O_CLOEXEC) != 0)
 		sudo_fatal("%s", U_("unable to create pipe"));
 	    io_buf_new(io_pipe[STDOUT_FILENO][0], STDOUT_FILENO,
-		log_stdout, read_callback, write_callback, &ec);
+		log_stdout, read_callback, write_callback, ec);
 	    io_fds[SFD_STDOUT] = io_pipe[STDOUT_FILENO][1];
 	}
     }
-    if (!sudo_isatty(STDERR_FILENO, &sb)) {
+    if (!fd_matches_tty(STDERR_FILENO, tty_sb, &sb)) {
 	if (!interpose[STDERR_FILENO]) {
 	    /* Not logging stderr, do not interpose. */
 	    sudo_debug_printf(SUDO_DEBUG_INFO,
-		"stderr not a tty, not logging");
-	    if (S_ISFIFO(sb.st_mode))
-		pipeline = true;
+		"stderr not user's tty, not logging");
 	    io_fds[SFD_STDERR] = dup(STDERR_FILENO);
 	    if (io_fds[SFD_STDERR] == -1)
 		sudo_fatal("dup");
 	} else {
 	    sudo_debug_printf(SUDO_DEBUG_INFO,
-		"stderr not a tty, creating a pipe");
+		"stderr not user's tty, creating a pipe");
 	    if (pipe2(io_pipe[STDERR_FILENO], O_CLOEXEC) != 0)
 		sudo_fatal("%s", U_("unable to create pipe"));
 	    io_buf_new(io_pipe[STDERR_FILENO][0], STDERR_FILENO,
-		log_stderr, read_callback, write_callback, &ec);
+		log_stderr, read_callback, write_callback, ec);
 	    io_fds[SFD_STDERR] = io_pipe[STDERR_FILENO][1];
 	}
     }
@@ -1273,16 +1303,19 @@ exec_pty(struct command_details *details,
     if (!sudo_term_copy(io_fds[SFD_USERTTY], io_fds[SFD_LEADER])) {
 	sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_ERRNO,
 	    "%s: unable to copy terminal settings to pty", __func__);
-	ec.foreground = false;
+	ec->foreground = false;
+    }
+    /* Start in raw mode unless the command will run in the background. */
+    cmnd_foreground = ec->foreground && !ISSET(details->flags, CD_EXEC_BG);
+    if (cmnd_foreground) {
+	if (sudo_term_raw(io_fds[SFD_USERTTY], 0))
+	    ec->term_raw = true;
     }
 
-    /* Start in raw mode unless part of a pipeline or backgrounded. */
-    if (ec.foreground) {
-	if (!pipeline && !ISSET(details->flags, CD_EXEC_BG)) {
-	    if (sudo_term_raw(io_fds[SFD_USERTTY], 0))
-		ec.term_raw = true;
-	}
-    }
+    sudo_debug_printf(SUDO_DEBUG_INFO,
+	"%s: follower: %d, stdin: %d, stdout: %d, stderr: %d", __func__,
+	io_fds[SFD_FOLLOWER], io_fds[SFD_STDIN], io_fds[SFD_STDOUT],
+	io_fds[SFD_STDERR]);
 
     /*
      * Block signals until we have our handlers setup in the parent so
@@ -1297,8 +1330,8 @@ exec_pty(struct command_details *details,
 	debug_return_bool(true);
     }
 
-    ec.monitor_pid = sudo_debug_fork();
-    switch (ec.monitor_pid) {
+    ec->monitor_pid = sudo_debug_fork();
+    switch (ec->monitor_pid) {
     case -1:
 	sudo_fatal("%s", U_("unable to fork"));
 	break;
@@ -1319,13 +1352,12 @@ exec_pty(struct command_details *details,
 	sudo_fatal_callback_deregister(pty_cleanup_hook);
 
 	/*                      
-	 * If stdin/stdout is not a tty, start command in the background
-	 * since it might be part of a pipeline that reads from /dev/tty.
-	 * In this case, we rely on the command receiving SIGTTOU or SIGTTIN
-	 * when it needs access to the controlling tty.
+	 * If stdin/stdout is not the user's tty, start the command in
+	 * the background since it might be part of a pipeline that reads
+	 * from /dev/tty.  In this case, we rely on the command receiving
+	 * SIGTTOU or SIGTTIN when it needs access to the controlling tty.
 	 */                                                              
-	exec_monitor(details, &oset, ec.foreground && !pipeline, sv[1],
-	    intercept_sv[1]);
+	exec_monitor(details, &oset, cmnd_foreground, sv[1], intercept_sv[1]);
 	cstat->type = CMD_ERRNO;
 	cstat->val = errno;
 	if (send(sv[1], cstat, sizeof(*cstat), 0) == -1) {
@@ -1333,6 +1365,7 @@ exec_pty(struct command_details *details,
                 "%s: unable to send status to parent", __func__);
 	}
 	_exit(EXIT_FAILURE);
+	/* NOTREACHED */
     }
 
     /*
@@ -1348,10 +1381,8 @@ exec_pty(struct command_details *details,
     /* Tell the monitor to continue now that the follower is closed. */
     cstat->type = CMD_SIGNO;
     cstat->val = 0;
-    while (send(sv[0], cstat, sizeof(*cstat), 0) == -1) {
-	if (errno != EINTR && errno != EAGAIN)
-	    sudo_fatal("%s", U_("unable to send message to monitor process"));
-    }
+    if (send(sv[0], cstat, sizeof(*cstat), 0) == -1)
+	sudo_fatal("%s", U_("unable to send message to monitor process"));
 
     /* Close the other end of the stdin/stdout/stderr pipes and socketpair. */
     if (io_pipe[STDIN_FILENO][0] != -1)
@@ -1376,14 +1407,14 @@ exec_pty(struct command_details *details,
      * Fill in exec closure, allocate event base, signal events and
      * the backchannel event.
      */
-    fill_exec_closure(&ec, cstat, details, user_details, evbase,
+    fill_exec_closure(ec, cstat, details, user_details, evbase,
 	sudo_pid, ppgrp, sv[0]);
 
     /* Create event and closure for intercept mode. */
     if (ISSET(details->flags, CD_INTERCEPT|CD_LOG_SUBCMDS)) {
-	ec.intercept = intercept_setup(intercept_sv[0], ec.evbase, details);
-	if (ec.intercept == NULL)
-	    terminate_command(ec.cmnd_pid, true);
+	ec->intercept = intercept_setup(intercept_sv[0], ec->evbase, details);
+	if (ec->intercept == NULL)
+	    terminate_command(ec->cmnd_pid, true);
     }
 
     /* Restore signal mask now that signal handlers are setup. */
@@ -1400,30 +1431,30 @@ exec_pty(struct command_details *details,
      * and pass output from leader to stdout and IO plugin.
      * Try to recover on ENXIO, it means the tty was revoked.
      */
-    add_io_events(&ec);
+    add_io_events(ec);
     do {
-	if (sudo_ev_dispatch(ec.evbase) == -1)
+	if (sudo_ev_dispatch(ec->evbase) == -1)
 	    sudo_warn("%s", U_("error in event loop"));
-	if (sudo_ev_got_break(ec.evbase)) {
-	    /* error from callback or monitor died */
+	if (sudo_ev_got_break(ec->evbase)) {
+	    /* error from callback */
 	    sudo_debug_printf(SUDO_DEBUG_ERROR, "event loop exited prematurely");
 	    /* XXX: no good way to know if we should terminate the command. */
-	    if (cstat->val == CMD_INVALID && ec.cmnd_pid != -1) {
+	    if (cstat->val == CMD_INVALID && ec->cmnd_pid != -1) {
 		/* no status message, kill command */
-		terminate_command(ec.cmnd_pid, true);
-		ec.cmnd_pid = -1;
+		terminate_command(ec->cmnd_pid, true);
+		ec->cmnd_pid = -1;
 		/* TODO: need way to pass an error to the sudo front end */
 		cstat->type = CMD_WSTATUS;
 		cstat->val = W_EXITCODE(1, SIGKILL);
 	    }
-	} else if (!sudo_ev_got_exit(ec.evbase)) {
+	} else if (!sudo_ev_got_exit(ec->evbase)) {
 	    switch (errno) {
 	    case ENXIO:
 	    case EIO:
 	    case EBADF:
 		/* /dev/tty was revoked, remove tty events and retry (once) */
 		if (evloop_retries == -1 && io_fds[SFD_USERTTY] != -1) {
-		    ev_free_by_fd(ec.evbase, io_fds[SFD_USERTTY]);
+		    ev_free_by_fd(ec->evbase, io_fds[SFD_USERTTY]);
 		    evloop_retries = 1;
 		}
 		break;
@@ -1431,11 +1462,14 @@ exec_pty(struct command_details *details,
 	}
     } while (evloop_retries-- > 0);
 
+    /* De-register cleanup hook, the pty is going away. */
+    sudo_fatal_callback_deregister(pty_cleanup_hook);
+
     /* Flush any remaining output, free I/O bufs and events, do logout. */
-    pty_finish(&ec, cstat);
+    pty_finish(ec, cstat);
 
     /* Free things up. */
-    free_exec_closure(&ec);
+    free_exec_closure(ec);
 
     debug_return_bool(true);
 }
@@ -1452,14 +1486,14 @@ sync_ttysize(struct exec_closure *ec)
 
     if (ioctl(io_fds[SFD_USERTTY], TIOCGWINSZ, &wsize) == 0) {
 	if (wsize.ws_row != ec->rows || wsize.ws_col != ec->cols) {
-	    sudo_debug_printf(SUDO_DEBUG_INFO, "%s: %hd x %hd -> %hd x %hd",
+	    sudo_debug_printf(SUDO_DEBUG_INFO, "%s: %d x %d -> %hd x %hd",
 		__func__, ec->rows, ec->cols, wsize.ws_row, wsize.ws_col);
 
 	    /* Log window change event. */
 	    log_winchange(ec, wsize.ws_row, wsize.ws_col);
 
 	    /* Update pty window size and send command SIGWINCH. */
-	    (void)ioctl(io_fds[SFD_LEADER], TIOCSWINSZ, &wsize);
+	    (void)ioctl(io_fds[SFD_LEADER], IOCTL_REQ_CAST TIOCSWINSZ, &wsize);
 	    killpg(ec->cmnd_pid, SIGWINCH);
 
 	    /* Update rows/cols. */
