@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 2009-2023 Todd C. Miller <Todd.Miller@sudo.ws>
+ * Copyright (c) 2009-2024 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -315,9 +315,10 @@ main(int argc, char *argv[], char *envp[])
     if (WIFSIGNALED(status)) {
 	struct sigaction sa;
 
-	if (WCOREDUMP(status))
-	    disable_coredump();
+	/* Make sure sudo doesn't dump core itself. */
+	disable_coredump();
 
+	/* Re-send the signal to the main sudo process. */
 	memset(&sa, 0, sizeof(sa));
 	sigemptyset(&sa.sa_mask);
 	sa.sa_handler = SIG_DFL;
@@ -506,8 +507,9 @@ static char **
 get_user_info(struct user_details *ud)
 {
     char *cp, **info, path[PATH_MAX];
-    size_t info_max = 32 + RLIM_NLIMITS;
+    size_t info_max = 33 + RLIM_NLIMITS;
     size_t i = 0, n;
+    dev_t ttydev;
     mode_t mask;
     struct passwd *pw;
     int ttyfd;
@@ -616,11 +618,17 @@ get_user_info(struct user_details *ud)
 	ud->cwd = info[i] + sizeof("cwd=") - 1;
     }
 
-    if (get_process_ttyname(path, sizeof(path)) != NULL) {
-	info[++i] = sudo_new_key_val("tty", path);
-	if (info[i] == NULL)
+    ttydev = get_process_ttyname(path, sizeof(path));
+    if (ttydev != (dev_t)-1) {
+	if (asprintf(&info[++i], "ttydev=%lld", (long long)ttydev) == -1)
 	    goto oom;
-	ud->tty = info[i] + sizeof("tty=") - 1;
+	/* The terminal device file may be missing in a chroot() jail. */
+	if (path[0] != '\0') {
+	    info[++i] = sudo_new_key_val("tty", path);
+	    if (info[i] == NULL)
+		goto oom;
+	    ud->tty = info[i] + sizeof("tty=") - 1;
+	}
     } else {
 	/* tty may not always be present */
 	if (errno != ENOENT)
