@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 2012-2022 Todd C. Miller <Todd.Miller@sudo.ws>
+ * Copyright (c) 2012-2024 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -23,7 +23,7 @@
 
 #include <config.h>
 
-/* Large files not supported by procfs.h on Solaris. */
+/* Large files may not be supported by procfs.h on Solaris. */
 #if defined(HAVE_STRUCT_PSINFO_PR_TTYDEV)
 # undef _FILE_OFFSET_BITS
 # undef _LARGE_FILES
@@ -94,16 +94,18 @@
 
 #if defined(sudo_kp_tdev)
 /*
- * Store the name of the tty to which the process is attached in name.
- * Returns name on success and NULL on failure, setting errno.
+ * Look up terminal device that the process is attached to and
+ * fill in its name, if available.  Sets name to the empty string
+ * if the device number cannot be mapped to a device name.
+ * Returns the tty device number on success and -1 on failure, setting errno.
  */
-char *
+dev_t
 get_process_ttyname(char *name, size_t namelen)
 {
     struct sudo_kinfo_proc *ki_proc = NULL;
     size_t size = sizeof(*ki_proc);
     int mib[6], rc, serrno = errno;
-    char *ret = NULL;
+    dev_t ttydev = (dev_t)-1;
     debug_decl(get_process_ttyname, SUDO_DEBUG_UTIL);
 
     /*
@@ -133,11 +135,13 @@ get_process_ttyname(char *name, size_t namelen)
     if (rc != -1) {
 	if ((dev_t)ki_proc->sudo_kp_tdev != (dev_t)-1) {
 	    errno = serrno;
-	    ret = sudo_ttyname_dev((dev_t)ki_proc->sudo_kp_tdev, name, namelen);
-	    if (ret == NULL) {
-		sudo_debug_printf(SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO|SUDO_DEBUG_ERRNO,
-		    "unable to map device number %lu to name",
-		    (unsigned long)ki_proc->sudo_kp_tdev);
+	    ttydev = (dev_t)ki_proc->sudo_kp_tdev;
+	    if (sudo_ttyname_dev(ttydev, name, namelen) == NULL) {
+		sudo_debug_printf(SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO,
+		    "unable to find terminal name for device %u, %u",
+		    (unsigned int)major(ttydev), (unsigned int)minor(ttydev));
+		if (namelen != 0)
+		    *name = '\0';
 	    }
 	}
     } else {
@@ -146,18 +150,21 @@ get_process_ttyname(char *name, size_t namelen)
     }
     free(ki_proc);
 
-    debug_return_str(ret);
+    debug_return_dev_t(ttydev);
 }
 #elif defined(HAVE_STRUCT_PSINFO_PR_TTYDEV)
 /*
- * Store the name of the tty to which the process is attached in name.
- * Returns name on success and NULL on failure, setting errno.
+ * Look up terminal device that the process is attached to and
+ * fill in its name, if available.  Sets name to the empty string
+ * if the device number cannot be mapped to a device name.
+ * Returns the tty device number on success and -1 on failure, setting errno.
  */
-char *
+dev_t
 get_process_ttyname(char *name, size_t namelen)
 {
-    char path[PATH_MAX], *ret = NULL;
+    dev_t ttydev = (dev_t)-1;
     struct psinfo psinfo;
+    char path[PATH_MAX];
     ssize_t nread;
     int fd, serrno = errno;
     debug_decl(get_process_ttyname, SUDO_DEBUG_UTIL);
@@ -168,16 +175,24 @@ get_process_ttyname(char *name, size_t namelen)
 	nread = read(fd, &psinfo, sizeof(psinfo));
 	close(fd);
 	if (nread == (ssize_t)sizeof(psinfo)) {
-	    dev_t rdev = (dev_t)psinfo.pr_ttydev;
+	    ttydev = (dev_t)psinfo.pr_ttydev;
 #if defined(_AIX) && defined(DEVNO64)
 	    if ((psinfo.pr_ttydev & DEVNO64) && sizeof(dev_t) == 4)
-		rdev = makedev(major64(psinfo.pr_ttydev), minor64(psinfo.pr_ttydev));
+		ttydev = makedev(major64(psinfo.pr_ttydev), minor64(psinfo.pr_ttydev));
 #endif
-	    if (rdev != (dev_t)-1) {
+	    /* On AIX, pr_ttydev is 0 (not -1) when no terminal is present. */
+	    if (ttydev != 0 && ttydev != (dev_t)-1) {
 		errno = serrno;
-		ret = sudo_ttyname_dev(rdev, name, namelen);
+		if (sudo_ttyname_dev(ttydev, name, namelen) == NULL) {
+		    sudo_debug_printf(SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO,
+			"unable to find terminal name for device %u, %u",
+			(unsigned int)major(ttydev), (unsigned int)minor(ttydev));
+		    if (namelen != 0)
+			*name = '\0';
+		}
 		goto done;
 	    }
+	    ttydev = (dev_t)-1;
 	}
     } else {
 	struct stat sb;
@@ -186,7 +201,14 @@ get_process_ttyname(char *name, size_t namelen)
 	/* Missing /proc/pid/psinfo file. */
 	for (i = STDIN_FILENO; i <= STDERR_FILENO; i++) {
 	    if (sudo_isatty(i, &sb)) {
-		ret = sudo_ttyname_dev(sb.st_rdev, name, namelen);
+		ttydev = sb.st_rdev;
+		if (sudo_ttyname_dev(ttydev, name, namelen) == NULL) {
+		    sudo_debug_printf(SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO,
+			"unable to find terminal name for device %u, %u",
+			(unsigned int)major(ttydev), (unsigned int)minor(ttydev));
+		    if (namelen != 0)
+			*name = '\0';
+		}
 		goto done;
 	    }
 	}
@@ -194,23 +216,25 @@ get_process_ttyname(char *name, size_t namelen)
     errno = ENOENT;
 
 done:
-    if (ret == NULL)
+    if (ttydev == (dev_t)-1)
 	sudo_debug_printf(SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO|SUDO_DEBUG_ERRNO,
 	    "unable to resolve tty via %s", path);
 
-    debug_return_str(ret);
+    debug_return_dev_t(ttydev);
 }
 #elif defined(__linux__)
 /*
- * Store the name of the tty to which the process is attached in name.
- * Returns name on success and NULL on failure, setting errno.
+ * Look up terminal device that the process is attached to and
+ * fill in its name, if available.  Sets name to the empty string
+ * if the device number cannot be mapped to a device name.
+ * Returns the tty device number on success and -1 on failure, setting errno.
  */
-char *
+dev_t
 get_process_ttyname(char *name, size_t namelen)
 {
     const char path[] = "/proc/self/stat";
+    dev_t ttydev = (dev_t)-1;
     char *cp, buf[1024];
-    char *ret = NULL;
     int serrno = errno;
     pid_t ppid = 0;
     ssize_t nread;
@@ -264,9 +288,15 @@ get_process_ttyname(char *name, size_t namelen)
 				 * signed int but the actual device number is an
 				 * unsigned int and dev_t is unsigned long long.
 				 */
-				dev_t tdev = (unsigned int)tty_nr;
+				ttydev = (unsigned int)tty_nr;
 				errno = serrno;
-				ret = sudo_ttyname_dev(tdev, name, namelen);
+				if (sudo_ttyname_dev(ttydev, name, namelen) == NULL) {
+				    sudo_debug_printf(SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO,
+					"unable to find terminal name for device %u, %u",
+					(unsigned int)major(ttydev), (unsigned int)minor(ttydev));
+				    if (namelen != 0)
+					*name = '\0';
+				}
 				goto done;
 			    }
 			    break;
@@ -288,7 +318,14 @@ get_process_ttyname(char *name, size_t namelen)
 	/* No parent pid found, /proc/self/stat is missing or corrupt. */
 	for (i = STDIN_FILENO; i <= STDERR_FILENO; i++) {
 	    if (sudo_isatty(i, &sb)) {
-		ret = sudo_ttyname_dev(sb.st_rdev, name, namelen);
+		ttydev = sb.st_rdev;
+		if (sudo_ttyname_dev(sb.st_rdev, name, namelen) == NULL) {
+		    sudo_debug_printf(SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO,
+			"unable to find terminal name for device %u, %u",
+			(unsigned int)major(ttydev), (unsigned int)minor(ttydev));
+		    if (namelen != 0)
+			*name = '\0';
+		}
 		goto done;
 	    }
 	}
@@ -298,23 +335,25 @@ get_process_ttyname(char *name, size_t namelen)
 done:
     if (fd != -1)
 	close(fd);
-    if (ret == NULL)
+    if (ttydev == (dev_t)-1)
 	sudo_debug_printf(SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO|SUDO_DEBUG_ERRNO,
 	    "unable to resolve tty via %s", path);
 
-    debug_return_str(ret);
+    debug_return_dev_t(ttydev);
 }
 #elif defined(HAVE_PSTAT_GETPROC)
 /*
- * Store the name of the tty to which the process is attached in name.
- * Returns name on success and NULL on failure, setting errno.
+ * Look up terminal device that the process is attached to and
+ * fill in its name, if available.  Sets name to the empty string
+ * if the device number cannot be mapped to a device name.
+ * Returns the tty device number on success and -1 on failure, setting errno.
  */
-char *
+dev_t
 get_process_ttyname(char *name, size_t namelen)
 {
-    struct pst_status pst;
-    char *ret = NULL;
+    dev_t ttydev = (dev_t)-1;
     int rc, serrno = errno;
+    struct pst_status pst;
     debug_decl(get_process_ttyname, SUDO_DEBUG_UTIL);
 
     /*
@@ -326,26 +365,32 @@ get_process_ttyname(char *name, size_t namelen)
     if (rc != -1 || errno == EOVERFLOW) {
 	if (pst.pst_term.psd_major != -1 && pst.pst_term.psd_minor != -1) {
 	    errno = serrno;
-	    ret = sudo_ttyname_dev(makedev(pst.pst_term.psd_major,
-		pst.pst_term.psd_minor), name, namelen);
+	    ttydev = makedev(pst.pst_term.psd_major, pst.pst_term.psd_minor);
+	    if (sudo_ttyname_dev(ttydev, name, namelen) == NULL) {
+		sudo_debug_printf(SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO,
+		    "unable to find terminal name for device %u, %u",
+		    (unsigned int)pst.pst_term.psd_major,
+		    (unsigned int)pst.pst_term.psd_minor);
+		if (namelen != 0)
+		    *name = '\0';
+	    }
 	    goto done;
 	}
     }
+    sudo_debug_printf(SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO|SUDO_DEBUG_ERRNO,
+	"unable to resolve tty via pstat");
     errno = ENOENT;
 
 done:
-    if (ret == NULL)
-	sudo_debug_printf(SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO|SUDO_DEBUG_ERRNO,
-	    "unable to resolve tty via pstat");
 
-    debug_return_str(ret);
+    debug_return_dev_t(ttydev);
 }
 #else
 /*
- * Store the name of the tty to which the process is attached in name.
- * Returns name on success and NULL on failure, setting errno.
+ * Look up terminal device that the process is attached to and fill in name.
+ * Returns the tty device number on success and -1 on failure, setting errno.
  */
-char *
+dev_t
 get_process_ttyname(char *name, size_t namelen)
 {
     struct stat sb;
@@ -365,38 +410,14 @@ get_process_ttyname(char *name, size_t namelen)
 	    sudo_debug_printf(
 		SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO|SUDO_DEBUG_ERRNO,
 		"unable to store tty from ttyname");
-	    debug_return_str(NULL);
+	    debug_return_dev_t(-1);
 	}
-	debug_return_str(name);
+	debug_return_dev_t(sb.st_rdev);
     }
 
     sudo_debug_printf(SUDO_DEBUG_WARN|SUDO_DEBUG_LINENO|SUDO_DEBUG_ERRNO,
 	"unable to resolve tty via ttyname");
     errno = ENOENT;
-    debug_return_str(NULL);
+    debug_return_dev_t((dev_t)-1);
 }
 #endif
-
-/*
- * Like isatty(3) but stats the fd and stores the result in sb.
- * Only calls isatty(3) if fd is a character special device.
- * Returns true if a tty, else returns false and sets errno.
- */
-bool
-sudo_isatty(int fd, struct stat *sb)
-{
-    bool ret = false;
-    debug_decl(sudo_isatty, SUDO_DEBUG_EXEC);
-
-    if (fstat(fd, sb) == 0) {
-        if (!S_ISCHR(sb->st_mode)) {
-            errno = ENOTTY;
-        } else {
-            ret = isatty(fd) == 1;
-        }
-    } else {
-	/* Always initialize sb. */
-	memset(sb, 0, sizeof(*sb));
-    }
-    debug_return_bool(ret);
-}
